<?php
/*
 * Copyright(c) 2015 PAYDESIGN, Inc. All rights reserved.
 * http://www.paydesign.co.jp/
 */

namespace Plugin\PayDesign\Controller;

use Eccube\Application;
use Eccube\Common\Constant;
use Plugin\PayDesign\Controller\Util\PluginUtil;
use Plugin\PayDesign\Form\Type\ConfigType;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Yaml\Yaml;

/**
 * Controller to handle module setting screen
 */
class ConfigController {

    public $app;
    public $const;

    /**
     * Edit config
     *
     * @param Application $app
     * @param Request $request
     * @param type $id
     * @return type
     */
    public function edit(Application $app, Request $request)
    {
        $this->app = $app;

        $objMdl =& PluginUtil::getInstance($this->app);
        $tpl_subtitle = $objMdl->getName();
        $objMdl->install();

        // Get module code from dtb_plugin
        $self = Yaml::parse(__DIR__ . '/../config.yml');
        $Plugin = $this->app['eccube.repository.plugin']->findOneBy(array('code' => $self['code']));

        if (is_null($Plugin)) {
            $error = "例外エラー<br />プラグインが存在しません。";
            $error_title = 'エラー';
            return $this->app['view']->render('error.twig', compact('error', 'error_title'));
        }

        $PayDesignPlugin = $this->app['eccube.plugin.paydesign.repository.paydesign_plugin']->findOneBy(array('code' => $Plugin->getCode()));

        if (is_null($PayDesignPlugin)) {
            $error = "例外エラー<br />プラグインが存在しません。";
            $error_title = 'エラー';
            return $this->app['view']->render('error.twig', compact('error', 'error_title'));
        }

        $subData = $objMdl->getUserSettings();
        $serverIp = $_SERVER['SERVER_ADDR'];
        $Payments = $objMdl->getPaymentInfo();

        $configForm = new ConfigType($this->app, $subData);
        $form = $this->app['form.factory']->createBuilder($configForm)->getForm();

        if ('POST' === $this->app['request']->getMethod()) {
            $form->handleRequest($this->app['request']);
            if ($form->isValid()) {
                $formData = $form->getData();
                $this->app['orm.em']->getConnection()->beginTransaction();
                $this->saveConfigData($formData, $Payments);
                $this->app['orm.em']->getConnection()->commit();

                $app->addSuccess('admin.register.complete', 'admin');
                return $this->app->redirect($this->app['url_generator']->generate('plugin_PayDesign_config'));
            }
        }

        return $this->app['view']->render('PayDesign/View/admin/paydesign_config.twig',
            array(
                'form'         => $form->createView(),
                'tpl_subtitle' => $tpl_subtitle,
                'serverIp'     => $serverIp,
                'subData'      => $subData,
            ));
    }

    /**
     * Save subdata into plg_paydesign_plugin
     * Save payment selected into dtb_payment and plg_paydesign_payment_method
     * @param array $data
     * @param array payment method name $Payments
     */
    public function saveConfigData($data, $Payments)
    {
        // Regist SubData
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->registerUserSettings($data);

        // Delete all payment method at dtb dtb_gmo_payment_method
        $PayDesignPaymentRepo = $this->app['eccube.plugin.paydesign.repository.paydesign_payment_method'];
        $PayDesignPaymentRepo->setConfig($this->app['config']['PayDesign']['const']);
        // Get all payment Id of this module
        $listId = $PayDesignPaymentRepo->getPaymentByCode(true, $this->app);

        // チェックされた決済を登録
        $installedPayment = array();
        foreach ($data['payment'] as $paymentTypeId) {
            //インストールされていなければ新規作成
            $id = $this->savePayment($paymentTypeId, $Payments);
            $this->savePayDesignPayment($id, $paymentTypeId, $Payments);
            $installedPayment[] = $id;
        }

        // チェックされていない決済を削除
        if (!empty($listId)) {
            foreach ((array)$listId as $paymentId) {
                if (!in_array($paymentId["id"], $installedPayment)) {
                    $removePayDesignPaymentMethod = $this->app['eccube.plugin.paydesign.repository.paydesign_payment_method']->find($paymentId["id"]);
                    if (!empty($removePayDesignPaymentMethod)){
                        $removePayDesignPaymentMethod->setDelFlg(1);
                        $this->app['orm.em']->persist($removePayDesignPaymentMethod);
                    }
                    $removePayment = $this->app['eccube.repository.payment']->find($paymentId["id"]);
                    if (!empty($removePayment)){
                        $removePayment->setDelFlg(1);
                        $this->app['orm.em']->persist($removePayment);
                    }
                    $this->app['orm.em']->flush();
                }
            }
        }
        // dtb_page_lauoutにも登録
        $this->registPlgPayDesignPageLayout();
    }

    /**
     * Insert or update payment that user selected
     * @param integer $paymentTypeId
     * @param array $Payments
     * @return payment_id
     */
    public function savePayment($paymentTypeId, $Payments) {

        // get dtb_payment default information
        $objMdl =& PluginUtil::getInstance($this->app);
        $arrDefault = $objMdl->getDtbPaymentDefultInfo();

        //Get PayDesignPaymentMethod of this paymentTypeId without considering del_flg
        $Payment = $this->app['orm.em']->getRepository('\Plugin\PayDesign\Entity\PayDesignPaymentMethod')->getPaymentByType($paymentTypeId, true, $this->app);

        // If no such data exists, create a new one
        if (is_null($Payment)) {
            $Payment = $this->app['orm.em']->getRepository('\Eccube\Entity\Payment')->findOrCreate(0);
        }

        // Set dtb_payment data
        // rule_min のデフォルト値は0とする
        $Payment->setMethod($Payments[$paymentTypeId]);
        $Payment->setFixFlg(1);
        $Payment->setRuleMin(0);
        $Payment->setCharge($arrDefault['charge']);
        $Payment->setUpdateDate(new \DateTime());
        $Payment->setCreateDate(new \DateTime());
        $Payment->setDelFlg(0);

        // Regist dtb_payment
        $this->app['orm.em']->persist($Payment);
        $this->app['orm.em']->flush();
        return $Payment->getId();
    }

    /**
     * Insert or update plg_paydesign_payment_method
     * @param integer $id
     * @param integer $paymentId
     * @param array $Payments
     */
    public function savePayDesignPayment($id, $paymentTypeId, $Payments)
    {

        $objMdl =& PluginUtil::getInstance($this->app);
        $pluginCode = $objMdl->getCode(true);

        $PayDesignPayment = $this->app['orm.em']->getRepository('\Plugin\PayDesign\Entity\PayDesignPaymentMethod')->getPayDesignPayment('id', $id, true, $this->app);

        if (is_null($PayDesignPayment)) {
            // Create new payment
            $PayDesignPayment = $this->app['eccube.plugin.paydesign.repository.paydesign_payment_method']->findOrCreate(0);
        }

        $PayDesignPayment->setId($id);
        $PayDesignPayment->setMethod($Payments[$paymentTypeId]);
        $PayDesignPayment->setDelFlg(0);
        $PayDesignPayment->setUpdateDate(new \DateTime());
        $PayDesignPayment->setCreateDate(new \DateTime());
        $PayDesignPayment->setMemo03($paymentTypeId); // install/uninstall の判断に使用
        $PayDesignPayment->setCode($pluginCode);
        $this->app['orm.em']->persist($PayDesignPayment);
        $this->app['orm.em']->flush();
    }

    /**
     * Register paydesign plugin all page layout
     */
    public function registPlgPayDesignPageLayout()
    {
        $arrPayDesignShoppingConvenience = array(
            'page_name' => '商品購入/ペイデザインコンビニ決済 コンビニ選択',
            'url'       => 'paydesign_shopping_convenience'
        );
        $this->registPageLayout($arrPayDesignShoppingConvenience);

        $arrPayDesignShoppingConvenienceComplete = array(
            'page_name' => '商品購入/ペイデザインコンビニ決済 ご注文完了',
            'url'       => 'paydesign_shopping_convenience_complete'
        );
        $this->registPageLayout($arrPayDesignShoppingConvenienceComplete);

        $arrPayDesignMypageCredit = array(
            'page_name' => 'MYページ/クレジットカード情報編集（入力ページ）',
            'url'       => 'paydesign_mypage_credit'
        );
        $this->registPageLayout($arrPayDesignMypageCredit);

        $arrPayDesignMypageCreditComplete = array(
            'page_name' => 'MYページ/クレジットカード情報編集（完了ページ）',
            'url'       => 'paydesign_mypage_credit_complete'
        );
        $this->registPageLayout($arrPayDesignMypageCreditComplete);

        $arrPayDesignMypageCreditDeleteComplete = array(
            'page_name' => 'MYページ/クレジットカード情報削除（完了ページ）',
            'url'       => 'paydesign_mypage_credit_delete_complete'
        );
        $this->registPageLayout($arrPayDesignMypageCreditDeleteComplete);

        $arrPayDesignMypagePeriodic = array(
            'page_name' => 'MYページ/定期情報（一覧ページ）',
            'url'       => 'paydesign_mypage_periodic'
        );
        $this->registPageLayout($arrPayDesignMypagePeriodic);

        $arrPayDesignMypagePeriodicEdit = array(
            'page_name' => 'MYページ/定期情報（編集ページ）',
            'url'       => 'paydesign_mypage_periodic_edit'
        );
        $this->registPageLayout($arrPayDesignMypagePeriodicEdit);

        $arrPayDesignMypagePeriodicComplete = array(
            'page_name' => 'MYページ/定期情報（完了ページ）',
            'url'       => 'paydesign_mypage_periodic_complete'
        );
        $this->registPageLayout($arrPayDesignMypagePeriodicComplete);

        // 2016.4.14 Shopping/confirm を追加（ヘッダー・フッター表示対応）
        // この設定はEC-CUBE標準で一度追加されたのちに削除されている設定
        // 他のプラグインと重複する可能性がある設定なので注意
        $arrPayDesignShoppingConfirm = array(
            'page_name' => '商品購入/ご入力内容のご確認',
            'url'       => 'shopping_confirm',
            'file_name' => 'Shopping/index'
        );
        $this->registPageLayout($arrPayDesignShoppingConfirm);

    }

    /**
     * Register page layout
     */
    public function registPageLayout($arrPageLayoutInfo) {

        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $oldVersion = $objMdl->getEccubeOldVersion();

        $deviceTypeRepo = $this->app['orm.em']->getRepository('Eccube\Entity\Master\DeviceType');
        $pageLayoutRepo = $this->app['orm.em']->getRepository('Eccube\Entity\PageLayout');
        in_array(Constant::VERSION, $oldVersion) ? $pageLayoutRepo->setApp($this->app) : $pageLayoutRepo->setApplication($this->app);
        $deviceType = $deviceTypeRepo->find(10);
        $pageLayout = $pageLayoutRepo->findOneBy(array('url' => $arrPageLayoutInfo['url']));
        if (is_null($pageLayout)) {
            $pageLayout = $pageLayoutRepo->newPageLayout($deviceType);
        }
        $pageLayout->setCreateDate(new \DateTime());
        $pageLayout->setUpdateDate(new \DateTime());
        $pageLayout->setName($arrPageLayoutInfo['page_name']);
        $pageLayout->setUrl($arrPageLayoutInfo['url']);
        // file_name はデータがある場合のみ設定を行う
        if(!empty($arrPageLayoutInfo['file_name'])) {
            $pageLayout->setFileName($arrPageLayoutInfo['file_name']); }
        $pageLayout->setMetaRobots('noindex');
        $pageLayout->setEditFlg('2');
        $this->app['orm.em']->persist($pageLayout);
        $this->app['orm.em']->flush();
    }
}
?>
