<?php
/*
 * Copyright(c) 2015 PAYDESIGN, Inc. All rights reserved.
 * http://www.paydesign.co.jp/
 */

namespace Plugin\PayDesign\Form\Type;

use Plugin\PayDesign\Controller\Util\PluginUtil;
use Plugin\PayDesign\Controller\Util\CommonUtil;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormError;
use Symfony\Component\Form\FormEvents;
use Symfony\Component\Validator\Constraints as Assert;

class ConfigType extends AbstractType
{
    private $app;
    private $subData;

    public function __construct(\Eccube\Application $app, $subData = null)
    {
        $this->app = $app;
        $this->subData = $subData;
    }

    /**
     * Build config type form
     *
     * @param FormBuilderInterface $builder
     * @param array $options
     * @return type
     */
    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $objPluginUtil =& PluginUtil::getInstance($this->app);
        $objPluginUtil->install();
        $arrPaymentInfo = $objPluginUtil->getPaymentInfo();
        $arrConvenienceInfo = $objPluginUtil->getConvenienceInfo();

        $objCommonUtil = new CommonUtil();
        $checkFlgInfo = $objCommonUtil->getCheckFlgInfo();
        $arrConvLimitDate = $objCommonUtil->getDaySelectParameter('日');

        $builder
            ->add('payment', 'choice', array(
                'label' => 'ご利用決済サービス',
                'choices' => $arrPaymentInfo,
                'data' => $this->subData['payment'],
                'multiple' => true,
                'expanded' => true,
              //'required' => true,
              //'constraints' => array(
              //    new Assert\NotBlank(array('message' => '※ ご利用決済サービスが選択されていません。')),
              //),
            ))
            ->add('prefix_word', 'text', array(
                'label' => 'プリフィックスワード（通常注文用）',
                'attr' => array(
                    'class' => 'lockon_card_row',
                ),
                'data' => $this->subData['prefix_word'],
                'constraints' => array(
                    new Assert\NotBlank(array('message' => '※ プリフィックスワード（通常注文用）が入力されていません。')),
                    new Assert\Regex(array(
                                         'pattern' => '/\w/',
                                         'message' => '※ プリフィックスワード（通常注文用）はアルファベット、数字、アンダースコアで入力をしてください。',
                                          )),
                ),
            ))
            ->add('uid_prefix_word', 'text', array(
                'label' => 'プリフィックスワード（ユーザID用）',
                'attr' => array(
                    'class' => 'lockon_card_row',
                ),
                'data' => $this->subData['uid_prefix_word'],
                'constraints' => array(
                    new Assert\NotBlank(array('message' => '※ プリフィックスワード（ユーザID用）が入力されていません。')),
                    new Assert\Regex(array(
                                         'pattern' => '/\w/',
                                         'message' => '※ プリフィックスワード（ユーザID用）はアルファベット、数字、アンダースコアで入力をしてください。',
                                          )),
                ),
            ))
            ->add('credit_shop_code', 'text', array(
                'label' => '加盟店コード',
                'attr' => array(
                    'class' => 'lockon_card_row',
                ),
                'data' => $this->subData['credit_shop_code'],
            ))
            ->add('credit_shop_passwd', 'text', array(
                'label' => '加盟店パスワード',
                'attr' => array(
                    'class' => 'lockon_card_row',
                ),
                'data' => $this->subData['credit_shop_passwd'],
            ))
            ->add('credit_auto_confirm', 'choice', array(
                'label' => '売上自動確定',
                'choices' => $checkFlgInfo,
                'expanded' => true,
                'data' => $this->subData['credit_auto_confirm'],
            ))
            ->add('credit_uid_settle', 'choice', array(
                'label' => 'ユーザーID決済',
                'choices' => $checkFlgInfo,
                'expanded' => true,
                'data' => $this->subData['credit_uid_settle'],
            ))
            ->add('credit_periodic_buy', 'choice', array(
                'label' => '定期購入',
                'choices' => $checkFlgInfo,
                'expanded' => true,
                'data' => $this->subData['credit_periodic_buy'],
            ))
            ->add('change_permission_periodic_next_date', 'choice', array(
                'label' => '会員によるマイページからの次回注文日の変更を許可する',
                'choices' => $checkFlgInfo,
                'expanded' => true,
                'data' => $this->subData['change_permission_periodic_next_date'],
            ))
            ->add('change_permission_periodic_status', 'choice', array(
                'label' => '会員によるマイページからの申込状況の変更を許可する',
                'choices' => $checkFlgInfo,
                'expanded' => true,
                'data' => $this->subData['change_permission_periodic_status'],
            ))
            ->add('conv_shop_code', 'text', array(
                'label' => '加盟店コード',
                'attr' => array(
                    'class' => 'lockon_card_row',
                ),
                'data' => $this->subData['conv_shop_code'],
            ))
            ->add('conv_shop_passwd', 'text', array(
                'label' => '加盟店パスワード',
                'attr' => array(
                    'class' => 'lockon_card_row',
                ),
                'data' => $this->subData['conv_shop_passwd'],
            ))
            ->add('convenience', 'choice', array(
                'label' => 'ご利用コンビニ情報',
                'choices' => $arrConvenienceInfo,
                'data' => $this->subData['convenience'],
                'multiple' => true,
                'expanded' => true,
                'required' => true,
            ))
            ->add('convenience_limitdate', 'choice', array(
                'label' => 'コンビニ支払期限',
                'choices' => $arrConvLimitDate,
                'data' => $this->subData['convenience_limitdate'],
                'multiple'=> false,
                'expanded' => false,
                'required' => false,
                'empty_value' => 'コンビニ支払期限を選択してください',
                'empty_data' => '',
            ))
            // 複合Validation処理
            ->addEventListener(FormEvents::POST_SUBMIT, function ($event) {
                $form = $event->getForm();
                $data = $form->getData();
                $credit_flg = false;
                if( count($data['payment']) > 0 ) {
                    foreach( $data['payment'] as $key => $value ) {
                        // 選択した決済方法に応じてValidation処理を実施
                        switch($value) {
                            case 'credit':
                                $credit_flg = true;
                                if(empty($data['credit_shop_code']) || is_null($data['credit_shop_code'])) {
                                    $form['credit_shop_code']
                                        ->addError(new FormError('※ 加盟店コードが入力されていません。'));
                                }
                                if(empty($data['credit_shop_passwd']) || is_null($data['credit_shop_passwd'])) {
                                    $form['credit_shop_passwd']
                                        ->addError(new FormError('※ 加盟店パスワードが入力されていません。'));
                                }
                                break ;
                            case 'convenience':
                                if(empty($data['conv_shop_code']) || is_null($data['conv_shop_code'])) {
                                    $form['conv_shop_code']
                                        ->addError(new FormError('※ 加盟店コードが入力されていません。'));
                                }
                                if(empty($data['conv_shop_passwd']) || is_null($data['conv_shop_passwd'])) {
                                    $form['conv_shop_passwd']
                                        ->addError(new FormError('※ 加盟店パスワードが入力されていません。'));
                                }
                                if( count($data['convenience']) == 0) {
                                    $form['convenience']
                                        ->addError(new FormError('※ ご利用コンビニ情報選択してください。'));
                                }
                                if(empty($data['convenience_limitdate']) || is_null($data['convenience_limitdate'])) {
                                    $form['convenience_limitdate']
                                        ->addError(new FormError('※ コンビニ支払期限を選択してください。'));
                                }
                                break ;
                            default:
                                break ;
                        }
                    }
                    if( $credit_flg ) {
                        if($data['credit_periodic_buy'] == "true" && $data['credit_uid_settle'] == "false") {
                            $form['credit_periodic_buy']
                                ->addError(new FormError('※ 定期購入機能を利用する場合はユーザーID決済を必ず有効にしてください。'));
                        }
                    } else {
                        if($data['credit_periodic_buy'] == "true") {
                            $form['credit_periodic_buy']
                                ->addError(new FormError('※ 定期購入機能を利用する場合はクレジットカード決済を必ず有効にしてください。'));
                        }
                    }
                }
            })
            ->addEventSubscriber(new \Eccube\Event\FormEventSubscriber());
    }

    /**
     * {@inheritdoc}
     */
    public function getName()
    {
        return 'config';
    }
}
?>
