<?php
/*
 * Copyright(c) 2015 PAYDESIGN, Inc. All rights reserved.
 * http://www.paydesign.co.jp/
 */

namespace Plugin\PayDesign\Service\client;

require_once(__DIR__ . "/../../vendor/autoload.php");

use Plugin\PayDesign\Controller\Util\CommonUtil;
use Plugin\PayDesign\Controller\Util\PluginUtil;

/**
 * 決済モジュール 決済処理: クレジット決済処理
 */
class PAYDESIGN_PLG_DCHK_CREDIT extends PAYDESIGN_PLG_DCHK {

    protected $app;
    protected $const;

    /**
     * コンストラクタ
     *
     * @return void
     */
    function __construct(\Eccube\Application $app)
    {
        parent::__construct($app);
        $this->app = $app;
        $this->const = $app['config']['PayDesign']['const'];
    }

    /*
     * ユーザーID型クレジット決済要求実行
     */
    public function execSettlement($Order, $PaydesignCardInfo, $periodic_flg = false, $kakutei_flg = false) {

        // 初期化
        $err_message = "" ;
        $sendData = array();
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // Start
        $objMdl->printLog('クレジットカード決済：ユーザーID決済要求実行 Start');

        // 送信データ作成

        $order_id = $Order->getId(); // 設定されている受注ID
        $sid = $this->const['PDP_CARD_PREFIX_UID'].$userSettings['prefix_word'].$order_id; // 2017.1 UID使用を示す接頭文字を付与
        // FUKA情報(定期注文の場合は設定)
        $fuka_param = NULL;
        if($periodic_flg) { $fuka_param = $this->const['PDP_FUKA_PARAM_PERIODIC_PARAM']; }
        // 基本情報の構築(true は PASSも付加)
        $sendData = $this->getBasicSendData($sid, 'credit', true, $fuka_param);

        // 確定・与信の設定を取得(確定の場合は$kakutei_paramに1を設定)
        // $kakutei_flg が true の場合は確定パラメータをセットする ... $kakutei_flg が true は定期注文作成時
        $kakutei_param = 0;
        if( ($userSettings['credit_auto_confirm'] == 'true') || $kakutei_flg ) { $kakutei_param = 1; }
        // クレジットカード情報(ユーザーID決済の場合は true, 引数 $ok_url, $rt_url部分は不要)
        $sendData = array_merge($sendData, $this->getCreditSendData($kakutei_param, $PaydesignCardInfo->getCardSid(), "", "", true));

        // 注文情報(ユーザーID決済の場合は true)
        $sendData = array_merge($sendData, $this->getOrderSendData($Order, true));

        // 登録のみ商品が含まれている場合等を考慮し、必要に応じて金額調整を行う(定期注文の時は処理不要)
        if( !$periodic_flg ) {
            $creditSettlementPattern = 0;
            $creditSettlementPattern = $this->getCreditSettlementPattern($Order, 1);
            // 2がかえってくるはず

            if( $creditSettlementPattern == 4 ) {
                // 金額調整を行う
                $sendData['K1'] = $this->reCalcK1($Order, $sendData['K1']);
            }
        }

        // 送信（※受信も）処理実行
        $response_body = $this->sendRequest($this->const['PDP_CREDIT_SEND_URL'], $sendData);

        if( is_array($response_body) ) {
            // レスポンスを解析
            if( $response_body[1] == $sid ) {
                switch($response_body[0]) {
                    case $this->const['PDP_PAYMENT_RESPONSE_CODE_OK']:
                        // OK ... 正常処理（注文完了）
                        // カート初期化(削除)
                        $cartService = $this->app['eccube.service.cart'];
                        $cartService->clear()->save();
                        // ログ
                        $objMdl->printLog("ユーザーID型クレジット決済 注文完了処理（SID=".$sid."）");
                        // 終了処理
                        $this->changeOrderData($Order, $periodic_flg);
                        break;
                    case $this->const['PDP_PAYMENT_RESPONSE_CODE_ERR']:
                    case $this->const['PDP_PAYMENT_RESPONSE_CODE_PRERR']:
                        // ERROR/PRERROR
                        if( $response_body[2] == $this->const['PDP_PAYMENT_RESPONSE_CODE_EXPERR'] ) {
                            // 有効期限切れ
                            $err_message = '【有効期限エラー】登録されたカードの有効期限が切れています。お手数ですが別のカード利用したご購入をお願い致します。' ;
                        } else {
                            // 異常終了
                            $err_message  = "【エラー】".$response_body[2]."<br />\n" ;
                            $err_message .= "エラーが発生しております。上記内容をご確認ください。";
                        }
                        $objMdl->printLog($err_message);
                        break;
                    default:
                        $err_message  = "【エラー】".$response_body[2]."<br />\n" ;
                        $err_message .= "お手数ですが「登録済みのカード情報を利用する」のチェックを外してからもう一度お試しください。";
                        $objMdl->printLog($err_message);
                        break;
                }
            } else {
                $err_message = '【注文ID不一致】正しい結果を取得する事が出来ませんでした。お手数ですが店舗担当者にご確認をお願い致します。（'.$sid.'/'.$response_body[1].'）';
                $objMdl->printLog($err_message);
            }
        } else {
            $err_message = '【不正エラー】不正なレスポンスを取得しました。お手数ですが店舗担当者にご確認をお願い致します。' ;
            $objMdl->printLog($err_message);
        }

        // End
        $objMdl->printLog('クレジットカード決済：ユーザーID決済要求実行 End');

        return $err_message ;
    }

    /*
     * 支払方法にクレジット決済選択 => 決済する必要がないパターン処理
     */
    public function execNoSettlement($Order)
    {
        // 決済処理をしないため、決済側の戻りが無いので、受注データの更新はここで実行する

        // プラグインユーティリティオブジェクトの作成
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        $objMdl->printLog("【非決済処理】受注データ更新・定期カードデータ登録処理開始");

        // 受注情報が存在しているか確認
        if(is_null($Order)) {
            $objMdl->printLog("【非決済処理】受注データ更新・定期カードデータ登録処理：受注情報取得失敗");
            return;
        }

        // 受注ステータスを新規登録に更新
        $now = new \DateTime();
        $Order
            ->setPaymentDate($now)
            ->setOrderDate($now)
            ->setUpdateDate($now)
            ->setOrderStatus($this->app['eccube.repository.order_status']->find($this->app['config']['order_new']));
        $this->app['orm.em']->persist($Order);
        $this->app['orm.em']->flush();

        /*
         * 通常と定期用カード情報を明確に分ける仕様にしたので、コピー処理は不要とする(2016.2.23)
        // 定期カード情報が登録されてなければ、通常カード情報を定期カード情報に登録する
        if( !is_null($Order->getCustomer()) ) {
            // 注文商品構成に定期商品が含まれているかどうか
            $periodic_flg = false;
            $periodic_flg = $this->getProductPattern($Order);
            if( $periodic_flg ) {
                $customer_id = $Order->getCustomer()->getId();
                $this->copyCardInfoToPeriodic($customer_id);
            }
        }
        */

        // カート初期化(削除)
        $cartService = $this->app['eccube.service.cart'];
        $cartService->clear()->save();

        $objMdl->printLog("【非決済処理】受注データ更新・定期カードデータ登録処理終了");

        return;
    }

    /*
     * （決済用）クレジットリンク先URL作成（非会員のケースもある）
     */
    public function getLinkUrl() {

        // 初期化
        $sendData = array();
        $link_url = "" ;
        $ok_url = $this->const['PDP_CREDIT_OK_URL'] ;
        $rt_url = $this->const['PDP_CREDIT_RT_URL'] ;

        // プラグイン設定情報の取得
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // 注文情報の取得
        if( !is_null($this->app['eccube.service.cart']->getPreOrderId()) ) {
            $Order = $this->app['eccube.repository.order']->findOneBy(array('pre_order_id' => $this->app['eccube.service.cart']->getPreOrderId()));
        } else {
            // pre_order_id が無い場合、セッション情報に保存したorder_idからの取得を試みる
            $session = $this->app['request']->getSession();
            if( !is_null($session->get('eccube.plugin.paydesign.orderId')) ) {
                $Order = $this->app['eccube.repository.order']->find($session->get('eccube.plugin.paydesign.orderId'));
            }
        }
        $order_id = $Order->getId(); // 設定されている受注ID
        $card_order_sid = $userSettings['prefix_word'].$order_id;

        // 定期商品が含まれているかどうか
        $periodic_flg = false;
        if( !is_null($Order) ) { $periodic_flg = $this->checkPeriodicProducts($Order); }

        // 顧客ID取得（非会員の場合もある）
        $customer_id = NULL;
        if ( !is_null($Order->getCustomer()) ) {
            $customer_id = $Order->getCustomer()->getId(); // 設定されている顧客ID

            // 対象注文内の定期商品の有無で取得カード情報を分ける
            $card_kind = $this->const['PDP_CARD_KIND_NORMAL'];
            if( $periodic_flg ) { $card_kind = $this->const['PDP_CARD_KIND_PERIODIC']; }
            $PayDesignCardInfo = $this->getCardInfoData($card_order_sid, $customer_id, $card_kind);
        }

        // 送信データ作成

        // 基本情報
        $sid = $card_order_sid;
        $sendData = $this->getBasicSendData($sid, 'credit');

        // 確定・与信の設定を取得(確定の場合は$kakutei_paramに1を設定)
        $kakutei_param = 0;
        if($userSettings['credit_auto_confirm'] == 'true') { $kakutei_param = 1; }

        // クレジットカード情報（※購入時にSIDを登録するのは定期商品を含んだ購入時のみ）
        $card_sid = NULL;
        if( !is_null($customer_id) && $periodic_flg ) { $card_sid = $PayDesignCardInfo->getCardSid(); }

        $sendData = array_merge($sendData, $this->getCreditSendData($kakutei_param, $card_sid, $ok_url, $rt_url));

        // 注文情報
        $sendData = array_merge($sendData, $this->getOrderSendData($Order));

        // 登録のみ商品が含まれている場合等を考慮し、必要に応じて金額調整を行う
        $creditSettlementPattern = 0;
        $creditSettlementPattern = $this->getCreditSettlementPattern($Order, 0);
        if( $creditSettlementPattern == 2 ) {
            // 金額調整を行う
            $sendData['K1'] = $this->reCalcK1($Order, $sendData['K1']);
        }

        // リダイレクト(リンク)URLの作成
        $link_url = $this->makeLinkUrl($this->const['PDP_CREDIT_LINK_URL'], $sendData);

        return $link_url;
    }

    /*
     * （注文時クレジットカード登録のみ）
     */
    public function getCardLinkOrderUrl($Order)
    {
        // 初期化
        $sendData = array();
        $link_url = "" ;
        $kakutei_param = 2;
        $ok_url = $this->const['PDP_CREDIT_OK_URL'] ;
        $rt_url = $this->const['PDP_CREDIT_RT_URL'] ;

        // プラグイン設定情報の取得
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // 定期商品が含まれているかどうか
        $periodic_flg = false;
        if( !is_null($Order) ) { $periodic_flg = $this->checkPeriodicProducts($Order); }

        // 会員の場合は、カード情報の取得
        $customer_id = NULL;
        $order_id = $Order->getId(); // 設定されている受注ID
        $card_order_sid = $userSettings['prefix_word'].$order_id;
        if ( !is_null($Order->getCustomer()) ) {
            $customer_id = $Order->getCustomer()->getId(); // 設定されている顧客ID
            // 対象注文内の定期商品の有無で取得カード情報を分ける
            $card_kind = $this->const['PDP_CARD_KIND_NORMAL'];
            if( $periodic_flg ) { $card_kind = $this->const['PDP_CARD_KIND_PERIODIC']; }
            $PayDesignCardInfo = $this->getCardInfoData($card_order_sid, $customer_id, $card_kind);
        }

        // 基本情報
        $sendData = $this->getBasicSendData($card_order_sid, 'credit');

        // クレジットカード情報
        $card_sid = "";
        if( !is_null($PayDesignCardInfo) ) { $card_sid = $PayDesignCardInfo->getCardSid(); }
        $sendData = array_merge($sendData, $this->getCreditSendData($kakutei_param, $card_sid, $ok_url, $rt_url) );

        // 注文情報
        $sendData = array_merge($sendData, $this->getOrderSendData($Order));

        // 金額は1円設定とする
        $sendData['K1'] = "1";

        // リダイレクト(リンク)URLの作成
        $link_url = $this->makeLinkUrl($this->const['PDP_CREDIT_LINK_URL'], $sendData);

        return $link_url;
    }

    /*
     * （マイページからのクレジットカード情報更新用）クレジットリンク先URL作成
     */
    public function getCardLinkUrl($param) {

        // 初期化
        $sendData = array();
        $link_url = "" ;
        $kakutei_param = 2;
        $ok_url = $this->const['PDP_CREDIT_CARD_UPDATE_OK_URL'] ;
        $rt_url = $this->const['PDP_CREDIT_CARD_UPDATE_RT_URL'] ;

        // カード更新用注文データ作成
        $PayDesignCardOrder = $this->getCardOrderData($this->app['user']->getId(), $param); 

        // カード登録情報の取得
        $PayDesignCardInfo = $this->getCardInfoData($PayDesignCardOrder->getCardOrderSid(),
                                                               $this->app['user']->getId(), $param);

        // 送信データ作成

        // 基本情報
        $sendData = $this->getBasicSendData($PayDesignCardOrder->getCardOrderSid(), 'credit');

        // クレジットカード情報
        $sendData = array_merge($sendData, $this->getCreditSendData($kakutei_param,
                                                  $PayDesignCardInfo->getCardSid(), $ok_url, $rt_url) );

        // (カード更新用)注文情報
        $sendData = array_merge($sendData, $this->getCardOrderSendData($this->app['user']->getId(), $param));

        // リダイレクト(リンク)URLの作成
        $link_url = $this->makeLinkUrl($this->const['PDP_CREDIT_LINK_URL'], $sendData);

        return $link_url;
    }

    /*
     * ユーザーID型クレジットカードデータ削除要求実行
     */
    public function execCardInfoDelete($customer_id, $param) {

        // 初期化
        $err_message = "";
        $card_kind_word = "";
        $sendData = array();

        // プラグイン設定情報の取得
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // カード種別文言設定
        if( $param == $this->const['PDP_CARD_KIND_NORMAL'] ) {
            $card_kind_word = "通常";
        } else {
            $card_kind_word = "定期";
        }

        // Start
        $objMdl->printLog('クレジットカード決済：ユーザーID決済カード情報削除要求実行 Start');

        // カード情報の取得
        $PayDesignCardInfo = NULL;
        $PayDesignCardInfo = $this->app['eccube.plugin.paydesign.repository.paydesign_card_info']->findOneBy(array('customer_id' => $customer_id, 'card_kind' => $param));
        if( !is_null($PayDesignCardInfo) ) {

            // 送信データ作成(カード削除用情報)
            $sendData = $this->getCardInfoDeleteSendData($PayDesignCardInfo->getCardSid());

            // 送信（※受信も）処理実行
            $response_body = $this->sendRequest($this->const['PDP_CREDIT_DELETE_URL'], $sendData);

            if( is_array($response_body) ) {
                if( strstr($response_body[0], $this->const['PDP_RESPONSE_CODE_NG'] ) ) {
                    $err_message = '【削除エラー】対象のカード情報が存在しませんでした。お手数ですが店舗担当者にご確認をお願い致します。（対象コード：'.$sendData['IP_USER_ID'].'）' ;
                } elseif( strstr($response_body[0], $this->const['PDP_RESPONSE_CODE_ERR'] ) ) {
                    $err_message = '【エラー】エラーが発生いたしました。お手数ですが店舗担当者にご確認をお願い致します。（対象コード：'.$sendData['IP_USER_ID'].'）' ;
                }
            } else {
                $err_message = '【不正エラー】不正なレスポンスを取得しました。お手数ですが店舗担当者にご確認をお願い致します。' ;
            }

        } else {
            $err_message  = $card_kind_word."注文用ユーザーID決済カード情報の取得に失敗しました。<br />\n";
            $err_message .= "登録されている".$card_kind_word."注文用カード情報を確認してください。";
            $objMdl->printLog($err_message);
        }

        // End
        $objMdl->printLog('クレジットカード決済：ユーザーID決済カード情報削除要求実行 End');

        return $err_message;
    }

    /*
     * ユーザーID型クレジットカードデータ削除(DB内の削除処理)
     */
    public function execCardInfoDeleteFromDB($customer_id, $param) {

        // プラグイン設定情報の取得
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();

        // カード情報の取得
        $PayDesignCardInfo = NULL;
        $PayDesignCardInfo = $this->app['eccube.plugin.paydesign.repository.paydesign_card_info']->findOneBy(array('customer_id' => $customer_id, 'card_kind' => $param));

        // DB内のカード情報削除処理（del_flgをセット）
        if( !is_null($PayDesignCardInfo) ) {
            $now = new \DateTime();
            $card_id = $PayDesignCardInfo->getId();
            $sid = $PayDesignCardInfo->getCardSid();
            $PayDesignCardInfo
                ->setCardEnable(0)
                ->setUpdateDate($now)
                ->setDelFlg(1);
            $this->app['orm.em']->persist($PayDesignCardInfo);
            $this->app['orm.em']->flush();
            $objMdl->printLog('DB[card_id:',$card_id,', card_sid:'.$sid.']からクレジットカードデータを削除しました。');
        }

        // 不要となった処理だが、万一を想定し削除処理だけは残しておく(2016.2.23)
        // 削除したカード情報と同じUIDで別のカード情報が登録されている可能性もあるので、そちらも削除
        $another_param = 0;
        if( $param == $this->const['PDP_CARD_KIND_NORMAL'] ) {
            $another_param = $this->const['PDP_CARD_KIND_PERIODIC'];
        } else {
            $another_param = $this->const['PDP_CARD_KIND_NORMAL'];
        }
        $AnotherPayDesignCardInfo = NULL;
        $AnotherPayDesignCardInfo = $this->app['eccube.plugin.paydesign.repository.paydesign_card_info']->findOneBy(array('customer_id' => $customer_id, 'card_kind' => $another_param));
        if( !is_null($AnotherPayDesignCardInfo) ) {
            $another_card_id = $AnotherPayDesignCardInfo->getId();
            $another_sid = $AnotherPayDesignCardInfo->getCardSid();
            if( $sid == $another_sid ) {
                // 上記で削除した情報と同じ内容を保持していた通常or定期カード情報を削除する
                $AnotherPayDesignCardInfo
                    ->setCardEnable(0)
                    ->setUpdateDate($now)
                    ->setDelFlg(1);
                $this->app['orm.em']->persist($AnotherPayDesignCardInfo);
                $this->app['orm.em']->flush();
                $objMdl->printLog('DB[card_id:',$another_card_id,', card_sid:'.$another_sid.']からクレジットカードデータを削除しました。');
            }
        }
    }

    /*
     * クレジットカード更新用注文完了処理
     */
    public function changeCardOrderData($PayDesignCardOrder) {

        // 現在日時
        $now = new \DateTime();

        // 会員カード情報の card_order_sid を更新する
        $card_ip_user_id = $PayDesignCardOrder->getCardIpUserId();
        $PayDesignCardInfo = 
            $this->app['eccube.plugin.paydesign.repository.paydesign_card_info']->findOneBy(
                array( 'card_sid' => $card_ip_user_id,
            ));
        if( !is_null($PayDesignCardInfo) ) {
            $PayDesignCardInfo
                ->setCardOrderSid($PayDesignCardOrder->getCardOrderSid())
                ->setUpdateDate($now);
            $this->app['orm.em']->persist($PayDesignCardInfo);
            $this->app['orm.em']->flush();
        }

        // 注文ステータスを『終了』に変更する
        $PayDesignCardOrder
            ->setCardOrderStatus($this->const['PDP_CARD_ORDER_STATUS_FINISH'])
            ->setCreateDate($now)
            ->setUpdateDate($now)
            ->setDelFlg(0);
        $this->app['orm.em']->persist($PayDesignCardOrder);
        $this->app['orm.em']->flush();
    }

    /*
     * クレジットカード決済：売上確定実行
     */
    public function commitOrder($Order) {

        // 初期化
        $ret = true;
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // Start
        $objMdl->printLog('クレジットカード決済：売上確定実行 Start');

        // 拡張注文情報の取得
        $PaydesignOrderInfo =
            $this->app['eccube.plugin.paydesign.repository.paydesign_order_info']->findOneBy( array( 'order_id' => $Order->getId(), 'order_kind' => $this->const['PDP_ORDER_KIND_NORMAL']));

        // 送信データの作成
        $sendData = $this->getBasicSendData($PaydesignOrderInfo->getSid(), 'credit');

        // 送信（※受信も）処理実行
        $response_body = $this->sendRequest($this->const['PDP_CREDIT_COMMIT_URL'], $sendData);

        if( is_array($response_body) ) {
            // レスポンスを解析
            if( strstr($response_body[0],$PaydesignOrderInfo->getSid()) ) {
                // エラーメッセージの作成
                if( strstr($response_body[0], $this->const['PDP_RESPONSE_CODE_NG']) ) {
                    $ret = array();
                    // エラーコードの設定
                    if( strstr($response_body[1], $this->const['PDP_RESPONSE_CODE_STATUS']) ) {
                        $start = strlen($this->const['PDP_RESPONSE_CODE_STATUS']);
                        $length = strlen($response_body[1]) - strlen($this->const['PDP_RESPONSE_CODE_STATUS']) ;
                        $ret['ErrCode'] = mb_substr($response_body[1], $start, $length,"UTF-8");
                        $errStatus = 'PDP_CREDIT_STATUS_'.$ret['ErrCode'];
                        $ret['ErrInfo'] = '【結果NG】決済状態が『'.$this->const[$errStatus].'』のため売上確定出来ませんでした。決済管理画面をご確認ください。' ;
                    } else {
                        $ret = array();
                        $ret['ErrCode'] = '-';
                        $ret['ErrInfo'] = '【結果NG】与信状態が確定していません。注文情報または決済管理画面をご確認ください。';
                    }
                } elseif( strstr($response_body[0], $this->const['PDP_RESPONSE_CODE_ERR']) ) {
                    $ret = array();
                    $ret['ErrCode'] = '-';
                    $ret['ErrInfo'] = '【エラー】正しい結果が取得できませんでした。注文情報または決済管理画面をご確認ください。';
                }
            } else {
                $msg = '【SID不一致】正しい結果を取得する事が出来ませんでした。注文情報をご確認ください。';
                $this->setError($msg);
                $ret = false ;
            }
        } else {
            $ret = false ;
        }

        // OK の場合は入金日を更新する（ステータスは【新規受付】）
        // $ret が配列の場合はエラーなので、除外する
        if($ret && !is_array($ret) ) {
            $now = new \DateTime();
            $Order
                ->setPaymentDate($now)
                ->setUpdateDate($now)
                ->setOrderStatus($this->app['eccube.repository.order_status']->find($this->app['config']['order_new']));
            $this->app['orm.em']->persist($Order);
            $this->app['orm.em']->flush();

            // 定期カード情報が登録されてなければ、通常カード情報を定期カード情報に登録する
            if( !is_null($Order->getCustomer()) ) {
                // 注文商品構成に定期商品が含まれているかどうか
                $periodic_flg = false;
                $periodic_flg = $this->getProductPattern($Order);
                if( $periodic_flg ) {
                    $customer_id = $Order->getCustomer()->getId();
                    $this->copyCardInfoToPeriodic($customer_id);
                }
            }
        }

        return $ret;
    }

    /*
     * クレジットカード決済：与信取消実行
     */
    public function authCancelOrder($Order) {

        // 初期化
        $ret = true;
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // Start
        $objMdl->printLog('クレジットカード決済：与信取消実行 Start');

        // 拡張注文情報の取得
        $PaydesignOrderInfo =
            $this->app['eccube.plugin.paydesign.repository.paydesign_order_info']->findOneBy( array( 'order_id' => $Order->getId(), 'order_kind' => $this->const['PDP_ORDER_KIND_NORMAL']));

        // 送信データの作成(true は PASSも付加)
        $sendData = $this->getBasicSendData($PaydesignOrderInfo->getSid(), 'credit', true);
        // 'FUKA'は不要なので削除
        unset($sendData['FUKA']);

        // 送信（※受信も）処理実行
        $response_body = $this->sendRequest($this->const['PDP_CREDIT_AUTH_CANCEL_URL'], $sendData);

        if( is_array($response_body) ) {
            // レスポンスを解析
            if( strstr($response_body[0],$PaydesignOrderInfo->getSid()) ) {
                // エラーメッセージの作成
                if( strstr($response_body[0], $this->const['PDP_RESPONSE_CODE_NG']) ) {
                    $ret = array();
                    // エラーコードの設定
                    if( strstr($response_body[1], $this->const['PDP_RESPONSE_CODE_STATUS']) ) {
                        $start = strlen($this->const['PDP_RESPONSE_CODE_STATUS']);
                        $length = strlen($response_body[1]) - strlen($this->const['PDP_RESPONSE_CODE_STATUS']) ;
                        $ret['ErrCode'] = mb_substr($response_body[1], $start, $length,"UTF-8");
                        $errStatus = 'PDP_CREDIT_STATUS_'.$ret['ErrCode'];
                        $ret['ErrInfo'] = '【結果NG】決済状態が『'.$this->const[$errStatus].'』のため与信取消出来ませんでした。決済管理画面をご確認ください。' ;
                    } else {
                        $ret = array();
                        $ret['ErrCode'] = '-';
                        $ret['ErrInfo'] = '【結果NG】与信状態が確定していません。注文情報または決済管理画面をご確認ください。';
                    }
                } elseif( strstr($response_body[0], $this->const['PDP_RESPONSE_CODE_ERR']) ) {
                    $ret = array();
                    $ret['ErrCode'] = '-';
                    $ret['ErrInfo'] = '【エラー】正しい結果が取得できませんでした。注文情報または決済管理画面をご確認ください。';
                } elseif( strstr($response_body[0], $this->const['PDP_RESPONSE_CODE_TO']) ) {
                    $ret = array();
                    $ret['ErrCode'] = '-';
                    $ret['ErrInfo'] = '【期限切れ】対象注文は与信取消期限が切れています。注文情報または決済管理画面をご確認ください。';
                }
            } else {
                $msg = '【SID不一致】正しい結果を取得する事が出来ませんでした。注文情報をご確認ください。';
                $this->setError($msg);
                $ret = false ;
            }
        } else {
            $ret = false ;
        }

        // End
        $objMdl->printLog('クレジットカード決済：取消(返品)実行 End');

        return $ret;
    }

    /*
     * クレジットカード決済：取消(返品)実行
     */
    public function cancelOrder($Order) {

        // 初期化
        $ret = true;
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // Start
        $objMdl->printLog('クレジットカード決済：取消(返品)実行 Start');

        // 拡張注文情報の取得
        $PaydesignOrderInfo =
            $this->app['eccube.plugin.paydesign.repository.paydesign_order_info']->findOneBy( array( 'order_id' => $Order->getId(), 'order_kind' => $this->const['PDP_ORDER_KIND_NORMAL']));

        // 送信データの作成(true は PASSも付加)
        $sendData = $this->getBasicSendData($PaydesignOrderInfo->getSid(), 'credit', true);
        // 'FUKA'は不要なので削除
        unset($sendData['FUKA']);

        // 送信（※受信も）処理実行
        $response_body = $this->sendRequest($this->const['PDP_CREDIT_CANCEL_URL'], $sendData);

        if( is_array($response_body) ) {
            // レスポンスを解析
            if( strstr($response_body[0],$PaydesignOrderInfo->getSid()) ) {
                // エラーメッセージの作成
                if( strstr($response_body[0], $this->const['PDP_RESPONSE_CODE_NG']) ) {
                    $ret = array();
                    // エラーコードの設定
                    if( strstr($response_body[1], $this->const['PDP_RESPONSE_CODE_STATUS']) ) {
                        $start = strlen($this->const['PDP_RESPONSE_CODE_STATUS']);
                        $length = strlen($response_body[1]) - strlen($this->const['PDP_RESPONSE_CODE_STATUS']) ;
                        $ret['ErrCode'] = mb_substr($response_body[1], $start, $length,"UTF-8");
                        $errStatus = 'PDP_CREDIT_STATUS_'.$ret['ErrCode'];
                        $ret['ErrInfo'] = '【結果NG】決済状態が『'.$this->const[$errStatus].'』のため取消(返品)出来ませんでした。決済管理画面をご確認ください。' ;
                    } else {
                        $ret = array();
                        $ret['ErrCode'] = '-';
                        $ret['ErrInfo'] = '【結果NG】決済状態が確定していません。注文情報または決済管理画面をご確認ください。';
                    }
                } elseif( strstr($response_body[0], $this->const['PDP_RESPONSE_CODE_ERR']) ) {
                    $ret = array();
                    $ret['ErrCode'] = '-';
                    $ret['ErrInfo'] = '【エラー】正しい結果が取得できませんでした。注文情報または決済管理画面をご確認ください。';
                } elseif( strstr($response_body[0], $this->const['PDP_RESPONSE_CODE_TO']) ) {
                    $ret = array();
                    $ret['ErrCode'] = '-';
                    $ret['ErrInfo'] = '【期限切れ】対象注文は取消期限が切れています。注文情報または決済管理画面をご確認ください。';
                }
            } else {
                $msg = '【SID不一致】正しい結果を取得する事が出来ませんでした。注文情報をご確認ください。';
                $this->setError($msg);
                $ret = false ;
            }
        } else {
            $ret = false ;
        }

        // End
        $objMdl->printLog('クレジットカード決済：取消(返品)実行 End');

        return $ret;
    }

    /*
     * クレジットカード決済：決済状態(オーソリ)確認 => 決済状態確認に変更
     */
    public function getAuthStatusOrder($Order) {

        // 初期化
        $ret = 1;
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // Start
        $objMdl->printLog('クレジットカード決済：決済状態確認 Start');

        // 拡張注文情報の取得
        $PaydesignOrderInfo =
            $this->app['eccube.plugin.paydesign.repository.paydesign_order_info']->findOneBy( array( 'order_id' => $Order->getId(), 'order_kind' => $this->const['PDP_ORDER_KIND_NORMAL']));

        // 送信データの作成
        $sendData = $this->getBasicSendData($PaydesignOrderInfo->getSid(), 'credit');
        // 'FUKA'は不要なので削除
        unset($sendData['FUKA']);

        // 送信（※受信も）処理実行
        $response_body = $this->sendRequest($this->const['PDP_CREDIT_GET_AUTH_STATUS_ORDER_URL'], $sendData);

        if( is_array($response_body) ) {
            // レスポンスを解析
            if( strstr($response_body[0],$PaydesignOrderInfo->getSid()) ) {
                // エラーメッセージの作成
                if( strstr($response_body[0], $this->const['PDP_RESPONSE_CODE_NG']) ) {
                    // エラーコードの取得
                    if( strstr($response_body[1], $this->const['PDP_RESPONSE_CODE_STATUS']) ) {
                        $start = strlen($this->const['PDP_RESPONSE_CODE_STATUS']);
                        $length = strlen($response_body[1]) - strlen($this->const['PDP_RESPONSE_CODE_STATUS']) ;
                        $ret = mb_substr($response_body[1], $start, $length,"UTF-8");
                    } else {
                        $ret = array();
                        $ret['ErrCode'] = '-';
                        $ret['ErrInfo'] = '【エラー】オンライン処理を実行する事が出来ませんでした。注文情報または決済管理画面をご確認ください。';
                    }
                } elseif( strstr($response_body[0], $this->const['PDP_RESPONSE_CODE_ERR']) ) {
                    $ret = array();
                    $ret['ErrCode'] = '-';
                    $ret['ErrInfo'] = '【エラー】正しい結果が取得できませんでした。注文情報または決済管理画面をご確認ください。';
                }
            } else {
                $msg = '【SID不一致】正しい結果を取得する事が出来ませんでした。注文情報をご確認ください。';
                $this->setError($msg);
                $ret = 0 ;
            }
        } else {
            $ret = 0 ;
        }

        // End
        $objMdl->printLog('クレジットカード決済：決済状態確認 End');

        return $ret;
    }

    /*
     * クレジットカード決済：決済状態(確定)確認
     */
    public function getStatusOrder($Order) {

        // 初期化
        $ret = true;
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // Start
        $objMdl->printLog('クレジットカード決済：決済状態(確定)確認 Start');

        // 拡張注文情報の取得
        $PaydesignOrderInfo =
            $this->app['eccube.plugin.paydesign.repository.paydesign_order_info']->findOneBy( array( 'order_id' => $Order->getId(), 'order_kind' => $this->const['PDP_ORDER_KIND_NORMAL']));

        // 送信データの作成
        $sendData = $this->getBasicSendData($PaydesignOrderInfo->getSid(), 'credit');
        // 'FUKA'は不要なので削除
        unset($sendData['FUKA']);

        // 送信（※受信も）処理実行
        $response_body = $this->sendRequest($this->const['PDP_CREDIT_GET_STATUS_ORDER_URL'], $sendData);

        if( is_array($response_body) ) {
            // レスポンスを解析
            if( strstr($response_body[0],$PaydesignOrderInfo->getSid()) ) {
                // エラーメッセージの作成
                if( strstr($response_body[0], $this->const['PDP_RESPONSE_CODE_NG']) ) {
                    // エラーコードの設定
                    if( strstr($response_body[1], $this->const['PDP_RESPONSE_CODE_STATUS']) ) {
                        $start = strlen($this->const['PDP_RESPONSE_CODE_STATUS']);
                        $length = strlen($response_body[1]) - strlen($this->const['PDP_RESPONSE_CODE_STATUS']) ;
                        $err_code = mb_substr($response_body[1], $start, $length,"UTF-8");
                    }
                    // エラーコードが PDP_CREDIT_STATUS_CODE_CONFIRM 以外の場合のみエラー
                    if( $err_code != $this->const['PDP_CREDIT_STATUS_CODE_CONFIRM'] ) {
                        $ret = array();
                        $ret['ErrCode'] = $err_code;
                        $errStatus = 'PDP_CREDIT_STATUS_'.$ret['ErrCode'];
                        $ret['ErrInfo'] = '【結果NG】決済状態は『'.$this->const[$errStatus].'』でした。決済管理画面をご確認ください。' ;
                    }
                } elseif( strstr($response_body[0], $this->const['PDP_RESPONSE_CODE_ERR']) ) {
                    $ret = array();
                    $ret['ErrInfo'] = '【エラー】正しい結果が取得できませんでした。注文情報あるいは決済管理画面をご確認ください。';
                }
            } else {
                $msg = '【SID不一致】正しい結果を取得する事が出来ませんでした。注文情報をご確認ください。';
                $this->setError($msg);
                $ret = false ;
            }
        } else {
            $ret = false ;
        }

        // End
        $objMdl->printLog('クレジットカード決済：決済状態(確定)確認 End');

        return $ret;
    }

    /*
     * クレジットカード登録情報の取得
     */
    function getCardInfoData($card_order_sid, $customer_id, $card_kind) {

        // 初期化
        $PayDesignCardInfo = NULL;
        $PayDesignCardInfo = $this->app['eccube.plugin.paydesign.repository.paydesign_card_info']->findOneBy(array('customer_id' => $customer_id, 'card_kind' => $card_kind));

        if( is_null($PayDesignCardInfo) ) {
            // データが存在しなければ、作成する
            $now = new \DateTime();
            $PayDesignCardInfo = new \Plugin\PayDesign\Entity\PayDesignCardInfo() ;
            $PayDesignCardInfo
                ->setCustomerId($customer_id)
                ->setCardKind($card_kind)
                ->setCardEnable(0)
            //  ->setCardOrderSid($card_order_sid) ... 確定してないので保存しない
                ->setCreateDate($now)
                ->setUpdateDate($now)
                ->setDelFlg(0);
            $this->app['orm.em']->persist($PayDesignCardInfo);
            $this->app['orm.em']->flush();

            // 初期データ作成後にIP_USER_IDを作成する
            $objMdl =& PluginUtil::getInstance($this->app);
            $objMdl->install();
            $userSettings = $objMdl->getUserSettings();
            $card_sid = $userSettings['uid_prefix_word'].$PayDesignCardInfo->getId();
            $PayDesignCardInfo
                ->setCardSid($card_sid);
            $this->app['orm.em']->persist($PayDesignCardInfo);
            $this->app['orm.em']->flush();
       }

       return $PayDesignCardInfo;
    }

    /*
     * 【画面表示用】クレジットカード登録情報の取得
     */
    function getCardInfoDataForDisp( $customer_id, $card_kind ) {

        // 初期化
        $ResultCardInfo = array() ;

        // カード情報の取得
        $CardInfo =
            $this->app['eccube.plugin.paydesign.repository.paydesign_card_info']->findOneBy( array( 'card_kind' => $card_kind, 'customer_id' => $customer_id ) );

        // パラメータの設定
        $ResultCardInfo["card_num"] = "";
        $ResultCardInfo["card_limit"] = "";
        $ResultCardInfo["card_sid"] = "";
        $ResultCardInfo["card_enable"] = $this->const['PDP_DISABLE'];
        if( !is_null($CardInfo) ) {
            $ResultCardInfo["card_num"] = $CardInfo->getCardNum();
            $ResultCardInfo["card_limit"] = $CardInfo->getCardLimit();
            $ResultCardInfo["card_sid"] = $CardInfo->getCardOrderSid();
            if( $CardInfo->getCardEnable() > 0 ) { $ResultCardInfo["card_enable"] = $this->const['PDP_ENABLE']; }

        }

        return $ResultCardInfo;
    }

    /*
     * クレジットカード登録用注文情報の取得
     */
    function getCardOrderData($customer_id, $param) {

        // 初期化
        $PayDesignCardOrder = NULL;

        // IP_USER_ID用のプリフィックスを取得
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // セッション情報を取得
        $session = $this->app['request']->getSession();
        $card_order_id = $session->get('eccube.mypage.paydesign.card_order_id');

        // セッション情報が取得出来ればIDから受注情報を取得
        if( !is_null($card_order_id) && isset($card_order_id) ) {
            $PayDesignCardOrder = $this->app['eccube.plugin.paydesign.repository.paydesign_card_order']->findOneBy( array( 'id' => $card_order_id, 'card_order_status' => $this->const['PDP_CARD_ORDER_STATUS_NEW'] ) ); }

        if( !is_null($PayDesignCardOrder) && isset($PayDesignCardOrder) ) {
            // 有効な受注情報が残っていた
            // 受注情報を更新（処理中に更新する）
            $PayDesignCardOrder->setCardOrderStatus($this->const['PDP_CARD_ORDER_STATUS_PENDING']);
            $this->app['orm.em']->persist($PayDesignCardOrder);
            $this->app['orm.em']->flush();
            // 取得した情報のIDをセッションに登録
            $session->set('eccube.mypage.paydesign.card_order_id', $PayDesignCardOrder->getId());
        } else {
            // カード情報IDを取得
            $PayDesignCardInfo = $this->app['eccube.plugin.paydesign.repository.paydesign_card_info']->findOneBy(array('customer_id' => $customer_id, 'card_kind' => $param));
            // データが存在しなければ、作成する（ card_order_sid はこの時点では無視）
            if( is_null($PayDesignCardInfo) ) {
                $now = new \DateTime();
                $PayDesignCardInfo = new \Plugin\PayDesign\Entity\PayDesignCardInfo() ;
                $PayDesignCardInfo
                    ->setCustomerId($customer_id)
                    ->setCardKind($param)
                    ->setCardEnable(0)
                    ->setCreateDate($now)
                    ->setUpdateDate($now)
                    ->setDelFlg(0);
                $this->app['orm.em']->persist($PayDesignCardInfo);
                $this->app['orm.em']->flush();

                // 初期データ作成後にIP_USER_IDを作成する
                $card_sid = $userSettings['uid_prefix_word'].$PayDesignCardInfo->getId();
                $PayDesignCardInfo
                    ->setCardSid($card_sid);
                $this->app['orm.em']->persist($PayDesignCardInfo);
                $this->app['orm.em']->flush();
            }

            // 受注情報を新規作成
            $now = new \DateTime();
            $PayDesignCardOrder = new \Plugin\PayDesign\Entity\PayDesignCardOrder() ;
            $PayDesignCardOrder
                ->setCustomerId($customer_id)
                ->setCardKind($param)
                ->setCardOrderStatus($this->const['PDP_CARD_ORDER_STATUS_PENDING'])
                ->setCardIpUserId($PayDesignCardInfo->getCardSid())
                ->setCreateDate($now)
                ->setUpdateDate($now)
                ->setDelFlg(0);
            $this->app['orm.em']->persist($PayDesignCardOrder);
            $this->app['orm.em']->flush();

            // プレフィックスワードの取得
            $prefix_word = "";
            if( $param == $this->const['PDP_CARD_KIND_NORMAL'] ) {
                // 通常注文用クレジットカード情報のためのプレフィックスワード
                $prefix_word = $this->const['PDP_CARD_PREFIX_NORMAL'];
            } else {
                // 定期注文用クレジットカード情報のためのプレフィックスワード
                $prefix_word = $this->const['PDP_CARD_PREFIX_PERIODIC'];
            }

            // 初期データ作成後に card_order_sid を作成する
            $card_order_sid = $prefix_word.$PayDesignCardOrder->getId();
            $PayDesignCardOrder
                ->setCardOrderSid($card_order_sid);
            $this->app['orm.em']->persist($PayDesignCardOrder);
            $this->app['orm.em']->flush();

            // 作成した情報のIDをセッションに登録
            $session->set('eccube.mypage.paydesign.card_order_id', $PayDesignCardOrder->getId());
        }

        return $PayDesignCardOrder;
    }

    /*
     * 【送信用】クレジット決済専用パラメータ取得
     */
    function getCreditSendData($kakutei_param, $ip_param, $ok_url, $rt_url, $uid_flg = false) {

        // ログ
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // コンビニエンス決済の設定がある場合は、KIGENも設定する
        $limitdate = "";
        foreach( $userSettings['payment'] as $val ) {
             if( $val == "convenience" ) {
                 // 支払期限の計算と設定
                 $limitdate = date("Ymd",( time() + 86400 * $userSettings['convenience_limitdate']));
                 break;
             }
        }

        $msg  = "クレジット決済 専用パラメータ：\n" ;
        if( $uid_flg ) {
            $msg .= "　1.KAKUTEI    ... ".$kakutei_param."\n";
            $msg .= "　2.IP_USER_ID ... ".$ip_param."\n";
            $msg .= "  3.KIGEN      ... ".$limitdate."\n";
        } else {
            $msg .= "　1.STORE      ... ".$this->const['PDP_CREDIT_PAYMENT_METHOD_CODE']."\n";
            $msg .= "　2.KAKUTEI    ... ".$kakutei_param."\n";
            $msg .= "　3.IP_USER_ID ... ".$ip_param."\n";
            $msg .= "　4.OKURL      ... ".$this->app->url('top').$ok_url."\n";
            $msg .= "　5.RT         ... ".$this->app->url('top').$rt_url."\n";
            $msg .= "  6.KIGEN      ... ".$limitdate."\n";
        }
        $objMdl->printLog($msg);

        // 初期設定
        $arrRet = array (
            'STORE'      => $this->const['PDP_CREDIT_PAYMENT_METHOD_CODE'],
            'KAKUTEI'    => $kakutei_param,
            'IP_USER_ID' => $ip_param,
            'OKURL'      => $this->app->url('top').$ok_url,
            'RT'         => $this->app->url('top').$rt_url,
        );
        // 支払期限の設定がある場合は設定追加
        if( $limitdate != "" ) { $arrRet['KIGEN'] = $limitdate; }

        // ユーザーID決済の場合は不要なデータを削除
        if( $uid_flg ) {
            unset($arrRet['STORE']);
            unset($arrRet['OKURL']);
            unset($arrRet['RT']);
        }

        return $arrRet;
    }

    /*
     * 【送信用】クレジットカード更新用注文パラメータ設定
     */
    function getCardOrderSendData($customer_id, $param) {

        // 汎用ユーティリティ（オブジェクト）の作成
        $objCommonUtil = new CommonUtil();

        // 会員情報の取得
        $Customer = $this->app['eccube.repository.customer']->find($customer_id);

        // 会員データを取得
        $name01 = $objCommonUtil->toRegularString($Customer->getName01(), 10);
        $name02 = $objCommonUtil->toRegularString($Customer->getName02(), 10);
        $kana01 = $objCommonUtil->toRegularString($Customer->getKana01(), 10);
        $kana02 = $objCommonUtil->toRegularString($Customer->getKana02(), 10);
        $tel    = $Customer->getTel01().$Customer->getTel02().$Customer->getTel03();
        $email  = $Customer->getEmail();
        $zip01  = $Customer->getZip01();
        $zip02  = $Customer->getZip02();
        $addr01 = $objCommonUtil->toRegularString($Customer->getPref().$Customer->getAddr01(), 25);
        $addr02 = $objCommonUtil->toRegularString($Customer->getAddr02(), 25);

        // クレジットカード更新用注文専用パラメータ
        $ext_word = "通常";
        if( $param == $this->const['PDP_CARD_KIND_PERIODIC'] ) { $ext_word = "定期"; }
        $k1 = "1" ;
        $n1 = $objCommonUtil->toRegularProductString($ext_word."注文用：クレジットカード更新", 19);

        // ログ
        $objMdl =& PluginUtil::getInstance($this->app);
        $msg  = $ext_word."注文用：クレジットカード更新 注文パラメータ：\n" ;
        $msg .= "　 1.NAME1  ... ".$name01."\n";
        $msg .= "　 2.NAME2  ... ".$name02."\n";
        $msg .= "　 3.KANA1  ... ".$kana01."\n";
        $msg .= "　 4.KANA2  ... ".$kana02."\n";
        $msg .= "　 5.YUBIN1 ... ".$zip01."\n";
        $msg .= "　 6.YUBIN2 ... ".$zip02."\n";
        $msg .= "　 7.TEL    ... ".$tel."\n";
        $msg .= "　 8.ADR1   ... ".$addr01."\n";
        $msg .= "　 9.ADR2   ... ".$addr02."\n";
        $msg .= "　10.MAIL   ... ".$email."\n";
        $msg .= "　11.N1     ... ".$n1."\n";
        $msg .= "　12.K1     ... ".$k1;
        $objMdl->printLog($msg);

        // データを設定
        return $arrRet = array(
            'NAME1'  => $name01,
            'NAME2'  => $name02,
            'KANA1'  => $kana01,
            'KANA2'  => $kana02,
            'YUBIN1' => $zip01,
            'YUBIN2' => $zip02,
            'TEL'    => $tel,
            'ADR1'   => $addr01,
            'ADR2'   => $addr02,
            'MAIL'   => $email,
            'N1'     => $n1,
            'K1'     => $k1,
        );
    }

    /*
     * 登録のみ商品が含まれている場合の金額調整処理
     */
    function reCalcK1($Order, $now_price)
    {
        // 初期化
        $k1 = $now_price;
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();

        $objMdl->printLog("金額調整処理開始：調整前金額（".$k1."）");

        $Shippings = $Order->getShippings();
        foreach( $Shippings as $shipping ) {
            $ShipmentItems = $shipping->getShipmentItems();
            foreach( $ShipmentItems as $shipmentItem ) {
                $product_class_id = $shipmentItem->getProductClass()->getId();
                $ProductPeriodicInfo = $this->app['eccube.plugin.paydesign.repository.paydesign_product_periodic_info']->find($product_class_id);
                // 定期商品フラグを取得
                $periodic_flg = false;
                if( !is_null($ProductPeriodicInfo) ) {
                    if( $ProductPeriodicInfo->getPeriodicSettleMethod() > 1 ) { $periodic_flg = true; } }
                // 定期商品であれば処理を継続
                if( $periodic_flg ) {
                    // getTotalPrice = getPriceIncTax(税込単価) * getQuantity(数量)
                    $k1 = $k1 - $shipmentItem->getTotalPrice(); // 明示的に記述
                }
            }
        }

        // 0以下となった場合は、以上なので値を元の値に戻す
        if( $k1 <= 0 ) {
            $objMdl->printLog("金額調整処理以上：異常金額（".$k1."）");
            $k1 = $now_price;
        }

        $objMdl->printLog("金額調整処理終了：調整後金額（".$k1."）");

        return $k1;
    }

    /*
     * 【送信用】クレジットカード削除用パラメータ設定
     */
    function getCardInfoDeleteSendData($ip_user_id) {

        // 初期化
        $sendData = array() ;

        // クレジットデータ送信用の基本情報を取得
        $sendData = $this->getBasicSendData("dummy", 'credit', true);

        // 不要データの削除
        unset($sendData['SID']); 
        unset($sendData['FUKA']);

        // 削除対象のIP_USER_ID( = plg_paydesign_card_info の card_sid )を追加
        $sendData['IP_USER_ID'] = $ip_user_id;

        return $sendData;
    }

    /*
     * 引数の会員に紐づいた全てのカード情報の削除処理
     * マイページの退会処理と会員管理画面の削除処理で使用
     */
    function deleteCreditCardInfoAll($customer_id)
    {
        if($customer_id)
        {
            // 引数の会員に紐づいた全てのカード情報を取得
            $PayDesignCardInfos = $this->app['eccube.plugin.paydesign.repository.paydesign_card_info']->findBy(array('customer_id'=>$customer_id));
            if($PayDesignCardInfos)
            { 
                // 取得したカード情報の削除フラグをセットする
                foreach( $PayDesignCardInfos as $PayDesignCardInfo ) {
                    $now = new \DateTime();
                    $PayDesignCardInfo
                        ->setUpdateDate($now)
                        ->setDelFlg(1);
                    $this->app['orm.em']->persist($PayDesignCardInfo);
                }
                $this->app['orm.em']->flush();
            }
        }
    }

    /*
     * 対応する決済パターンの取得
     */
    function getCreditSettlementPattern($Order, $credit_uid_card_select_flg)
    {
        $creditSettlementPattern = 0;
        $productPattern = 0;
        $cardRegistPattern = 0;

        // 注文商品構成から購入パターンを取得する
        $productPattern = $this->getProductPattern($Order);

        // 登録されているカードのパターンの取得
        $cardRegistPattern = $this->getCardRegistPattern($Order);

        switch($cardRegistPattern) {
            case 1: // 通ク無・定ク無
            case 2: // 通ク有・定ク無
            case 3: // 通ク無・定ク有
            case 4: // 通ク有・定ク有
                switch($productPattern) {
                    case 1: // 通常商品も登録のみ商品も含まれている場合
                        if( $credit_uid_card_select_flg == 0 ) {
                            // ベースはリンク型決済
                            $creditSettlementPattern = 2;
                        } else {
                            // ベースはUID型決済
                            $creditSettlementPattern = 4;
                        }
                        break;
                    case 2: // 登録のみ商品だけの場合
                        if( $credit_uid_card_select_flg == 0 ) {
                            // ベースはリンク型決済
                            $creditSettlementPattern = 1;
                        } else {
                            // ベースはUID型決済
                            $creditSettlementPattern = 3;
                        }
                        break;
                    default:
                        break;
                }
                break ;
            default:
                break;
        }

        return $creditSettlementPattern;
    }

    /*
     * 登録されているカードのパターンの取得
     */
    function getCardRegistPattern($Order)
    {
        $cardRegistPattern = 0;
        $normal_flg = false;
        $periodic_flg = false;

        if( !is_null($Order->getCustomer()) ) {

            // 会員IDの取得
            $customer_id = $Order->getCustomer()->getId();

            if( $customer_id > 0 ) {
                // 通常カード情報の取得
                $PaydesignNormalCardInfo =
                    $this->app['eccube.plugin.paydesign.repository.paydesign_card_info']->findOneBy(
                        array( 'customer_id' => $customer_id,
                               'card_kind'   => $this->const['PDP_CARD_KIND_NORMAL'],
                    ));
                if( !is_null($PaydesignNormalCardInfo) ) {
                    if( $PaydesignNormalCardInfo->getCardEnable() == 1 ) { $normal_flg = true; } }

                // 定期カード情報の取得
                $PaydesignPeriodicCardInfo =
                    $this->app['eccube.plugin.paydesign.repository.paydesign_card_info']->findOneBy(
                        array( 'customer_id' => $customer_id,
                               'card_kind'   => $this->const['PDP_CARD_KIND_PERIODIC'],
                    ));
                if( !is_null($PaydesignPeriodicCardInfo) ) {
                    if( $PaydesignPeriodicCardInfo->getCardEnable() == 1 ) { $periodic_flg = true; } }
            }
        }

        // パターン毎にパラメータを設定(非会員の場合は1となる)
        if( !$normal_flg && !$periodic_flg ) { $cardRegistPattern = 1; } // 通ク無・定ク無
        if(  $normal_flg && !$periodic_flg ) { $cardRegistPattern = 2; } // 通ク有・定ク無
        if( !$normal_flg &&  $periodic_flg ) { $cardRegistPattern = 3; } // 通ク無・定ク有
        if(  $normal_flg &&  $periodic_flg ) { $cardRegistPattern = 4; } // 通ク有・定ク有

        return $cardRegistPattern;
    }

    /*
     * 通常カード情報を定期カード情報にコピーする
     */
    function copyCardInfoToPeriodic($customer_id)
    {
        // プラグインユーティリティオブジェクトの作成
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        // 通常カード情報の取得
        $PaydesignNormalCardInfo = NULL;
        $PaydesignNormalCardInfo =
            $this->app['eccube.plugin.paydesign.repository.paydesign_card_info']->findOneBy(
                array( 'customer_id' => $customer_id,
                       'card_kind'   => $this->const['PDP_CARD_KIND_NORMAL'],
            ));

        if( !is_null($PaydesignNormalCardInfo) ) {

            // 定期カード情報の取得
            $PaydesignPeriodicCardInfo = NULL;
            $PaydesignPeriodicCardInfo =
                $this->app['eccube.plugin.paydesign.repository.paydesign_card_info']->findOneBy(
                    array( 'customer_id' => $customer_id,
                           'card_kind'   => $this->const['PDP_CARD_KIND_PERIODIC'],
                ));

            if( is_null( $PaydesignPeriodicCardInfo ) ) {
                // 定期カード情報が無い場合にコピーする
                $now = new \DateTime();
                $PaydesignPeriodicCardInfo = new \Plugin\PayDesign\Entity\PayDesignCardInfo() ;
                $PaydesignPeriodicCardInfo
                    ->setCardSid($PaydesignNormalCardInfo->getCardSid())
                    ->setCustomerId($customer_id)
                    ->setCardKind($this->const['PDP_CARD_KIND_PERIODIC'])
                    ->setCardNum($PaydesignNormalCardInfo->getCardNum())
                    ->setCardLimit($PaydesignNormalCardInfo->getCardLimit())
                    ->setCardEnable($PaydesignNormalCardInfo->getCardEnable())
                    ->setCardOrderSid($PaydesignNormalCardInfo->getCardOrderSid())
                    ->setCreateDate($now)
                    ->setUpdateDate($now)
                    ->setDelFlg(0);
                $this->app['orm.em']->persist($PaydesignPeriodicCardInfo);
                $this->app['orm.em']->flush();
            }
        }

        return ;
    }
}
?>
