<?php
/*
 * Copyright(c) 2015 PAYDESIGN, Inc. All rights reserved.
 * http://www.paydesign.co.jp/
 */

namespace Plugin\PayDesign\Service\client;

require_once(__DIR__ . "/../../vendor/autoload.php");

use Plugin\PayDesign\Controller\Util\PluginUtil;

/**
 * 決済モジュール 定期処理:
 */
class PAYDESIGN_PLG_DCHK_PERIODIC extends PAYDESIGN_PLG_DCHK_CREDIT {

    protected $app;
    protected $const;

    /**
     * コンストラクタ
     *
     * @return void
     */
    function __construct(\Eccube\Application $app)
    {
        parent::__construct($app);
        $this->app = $app;
        $this->const = $app['config']['PayDesign']['const'];
    }

    /*
     * 【注文時】定期マスタ作成処理
     */
    public function createPeriodicInfo($Order) {

        // 初期化
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();

        $order_id = $Order->getId();
        $Shippings = $Order->getShippings();
        foreach( $Shippings as $shipping ) {
            $ShipmentItems = $shipping->getShipmentItems();
            foreach( $ShipmentItems as $shipmentItem ) {
                $product_class_id = $shipmentItem->getProductClass()->getId();
                $ProductPeriodicInfo = $this->app['eccube.plugin.paydesign.repository.paydesign_product_periodic_info']->find($product_class_id);
                // 定期商品フラグを取得
                $periodic_flg = false;
                if( !is_null($ProductPeriodicInfo) ) {
                    if( $ProductPeriodicInfo->getPeriodicFlg() > 0 ) { $periodic_flg = true; } }
                // 定期商品であれば処理を実行
                if( $periodic_flg ) {
                    $objMdl->printLog("定期マスタ作成開始:注文ID[".$order_id."]/商品規格ID[".$product_class_id."]");
                    $now = new \DateTime();
                    $now_day = date('d', strtotime($now->format('Y-m-d H:i:s')));
                    $periodicStatus = $objMdl->getPeriodicStatus();
                    $PayDesignPeriodicInfo = new \Plugin\PayDesign\Entity\PayDesignPeriodicInfo();
                    // データの作成
                    $arrPeriodicInfo = array();
                    $arrPeriodicInfo['periodic_status'] = $this->const['PDP_PERIODIC_STATUS_ACTIVE'];
                    $periodic_next_date = $this->createPeriodicNextDate($ProductPeriodicInfo, $now, $now_day,"", false);
                    $arrPeriodicInfo['periodic_next_date'] = $periodic_next_date->format('Y/m/d H:i:s');
                    $arrPeriodicInfo['periodic_interval'] = $ProductPeriodicInfo->getPeriodicInterval();
                    if( !is_null($ProductPeriodicInfo->getPeriodicCreateDay()) ) {
                        $arrPeriodicInfo['periodic_create_day'] = $ProductPeriodicInfo->getPeriodicCreateDay();
                    } else {
                        // 商品毎の次回注文基準日が存在せず、通常商品の場合は $now_day を設定する
                        // $now_day は商品注文完了日に等しい
                        if( $ProductPeriodicInfo->getPeriodicSettleMethod() != 2 ) {
                            if( $now_day == date('d',strtotime('last day of '.$now->format('Y-m'))) ) {
                                $arrPeriodicInfo['periodic_create_day'] = 31;
                            } else {
                                $arrPeriodicInfo['periodic_create_day'] = $now_day;
                            }
                        }
                    }
                    $arrPeriodicInfo['customer_id'] = $Order->getCustomer()->getId();
                    $arrPeriodicInfo['payment_id'] = $Order->getPayment()->getId();
                    $arrPeriodicInfo['payment_method'] = $Order->getPayment()->getMethod();
                    $arrPeriodicInfo['device_type_id'] = NULL;
                    if( !is_null($Order->getDeviceType()) ) {
                        $arrPeriodicInfo['device_type_id'] = $Order->getDeviceType()->getId(); }
                    $arrPeriodicInfo['message'] = $Order->getMessage();
                    $arrPeriodicInfo['shipping_country_id'] = NULL;
                    if( !is_null($shipping->getCountry()) ) {
                        $arrPeriodicInfo['shipping_country_id'] = $shipping->getCountry()->getId(); }
                    $arrPeriodicInfo['shipping_pref'] = $shipping->getPref()->getId();
                    $arrPeriodicInfo['shipping_name01'] = $shipping->getName01();
                    $arrPeriodicInfo['shipping_name02'] = $shipping->getName02();
                    $arrPeriodicInfo['shipping_kana01'] = $shipping->getKana01();
                    $arrPeriodicInfo['shipping_kana02'] = $shipping->getKana02();
                    $arrPeriodicInfo['shipping_company_name'] = $shipping->getCompanyName();
                    $arrPeriodicInfo['shipping_tel01'] = $shipping->getTel01();
                    $arrPeriodicInfo['shipping_tel02'] = $shipping->getTel02();
                    $arrPeriodicInfo['shipping_tel03'] = $shipping->getTel03();
                    $arrPeriodicInfo['shipping_fax01'] = $shipping->getFax01();
                    $arrPeriodicInfo['shipping_fax02'] = $shipping->getFax02();
                    $arrPeriodicInfo['shipping_fax03'] = $shipping->getFax03();
                    $arrPeriodicInfo['shipping_zip01'] = $shipping->getZip01();
                    $arrPeriodicInfo['shipping_zip02'] = $shipping->getZip02();
                    $arrPeriodicInfo['shipping_zipcode'] = $shipping->getZipcode();
                    $arrPeriodicInfo['shipping_addr01'] = $shipping->getAddr01();
                    $arrPeriodicInfo['shipping_addr02'] = $shipping->getAddr02();
                    $arrPeriodicInfo['shipping_delivery_name'] = $shipping->getShippingDeliveryName();
                    $arrPeriodicInfo['shipping_delivery_time'] = $shipping->getShippingDeliveryTime();
                    $arrPeriodicInfo['delivery_id'] = NULL;
                    if( !is_null($shipping->getDelivery()) ) {
                        $arrPeriodicInfo['delivery_id'] = $shipping->getDelivery()->getId(); }
                    $arrPeriodicInfo['time_id'] = NULL;
                    if( !is_null($shipping->getDeliveryTime()) ) {
                        $arrPeriodicInfo['time_id'] = $shipping->getDeliveryTime()->getId(); }
                    $arrPeriodicInfo['fee_id'] = NULL;
                    if( !is_null($shipping->getDeliveryFee()) ) {
                        $arrPeriodicInfo['fee_id'] = $shipping->getDeliveryFee()->getId(); }
                    $arrPeriodicInfo['subtotal'] = $Order->getSubtotal();
                    $arrPeriodicInfo['discount'] = $Order->getDiscount();
                    $arrPeriodicInfo['delivery_fee_total'] = $Order->getDeliveryFeeTotal();
                    $arrPeriodicInfo['charge'] = $Order->getCharge();
                    $arrPeriodicInfo['tax'] = $Order->getTax();
                    $arrPeriodicInfo['total'] = $Order->getTotal();
                    $arrPeriodicInfo['payment_total'] = $Order->getPaymentTotal();
                    $arrPeriodicInfo['product_id'] = $shipmentItem->getProduct()->getId();
                    $arrPeriodicInfo['product_class_id'] = $shipmentItem->getProductClass()->getId();
                    $arrPeriodicInfo['product_name'] = $shipmentItem->getProductName();
                    $arrPeriodicInfo['product_code'] = $shipmentItem->getProductCode();
                    $arrPeriodicInfo['class_name1'] = $shipmentItem->getClassName1();
                    $arrPeriodicInfo['class_name2'] = $shipmentItem->getClassName2();
                    $arrPeriodicInfo['class_category_name1'] = $shipmentItem->getClassCategoryName1();
                    $arrPeriodicInfo['class_category_name2'] = $shipmentItem->getClassCategoryName2();
                    $arrPeriodicInfo['price'] = $shipmentItem->getPrice();
                    if( !is_null($ProductPeriodicInfo->getPeriodicPrice()) ) {
                        $arrPeriodicInfo['price'] = $ProductPeriodicInfo->getPeriodicPrice(); }
                    $arrPeriodicInfo['quantity'] = $shipmentItem->getQuantity();
                    $arrPeriodicInfo['note'] = $Order->getNote();
                    $arrPeriodicInfo['base_order_id'] = $Order->getId();
                    $arrPeriodicInfo['base_shipping_id'] = $shipmentItem->getId();
                    $arrPeriodicInfo['base_item_id'] = $shipmentItem->getId();
                    $arrPeriodicInfo['create_date'] = $now->format('Y/m/d H:i:s');
                    $arrPeriodicInfo['update_date'] = $now->format('Y/m/d H:i:s');
                    $arrPeriodicInfo['del_flg'] = 0;
                    $objMdl->printLog("定期マスタ登録内容");
                    $objMdl->printLog($arrPeriodicInfo);
                    // データの設定と登録
                    $PayDesignPeriodicInfo
                        ->setPeriodicStatus($arrPeriodicInfo['periodic_status'])
                        ->setPeriodicNextDate($periodic_next_date)
                        ->setPeriodicInterval($arrPeriodicInfo['periodic_interval'])
                        ->setPeriodicCreateDay($arrPeriodicInfo['periodic_create_day'])
                        ->setCustomer($Order->getCustomer())
                        ->setCustomerId($arrPeriodicInfo['customer_id'])
                        ->setPaymentId($arrPeriodicInfo['payment_id'])
                        ->setPaymentMethod($arrPeriodicInfo['payment_method'])
                        ->setDeviceTypeId($arrPeriodicInfo['device_type_id'])
                        ->setMessage($arrPeriodicInfo['message'])
                        ->setShippingCountryId($arrPeriodicInfo['shipping_country_id'])
                        ->setShippingPref($arrPeriodicInfo['shipping_pref'])
                        ->setShippingName01($arrPeriodicInfo['shipping_name01'])
                        ->setShippingName02($arrPeriodicInfo['shipping_name02'])
                        ->setShippingKana01($arrPeriodicInfo['shipping_kana01'])
                        ->setShippingKana02($arrPeriodicInfo['shipping_kana02'])
                        ->setShippingCompanyName($arrPeriodicInfo['shipping_company_name'])
                        ->setShippingTel01($arrPeriodicInfo['shipping_tel01'])
                        ->setShippingTel02($arrPeriodicInfo['shipping_tel02'])
                        ->setShippingTel03($arrPeriodicInfo['shipping_tel03'])
                        ->setShippingFax01($arrPeriodicInfo['shipping_fax01'])
                        ->setShippingFax02($arrPeriodicInfo['shipping_fax02'])
                        ->setShippingFax03($arrPeriodicInfo['shipping_fax03'])
                        ->setShippingZip01($arrPeriodicInfo['shipping_zip01'])
                        ->setShippingZip02($arrPeriodicInfo['shipping_zip02'])
                        ->setShippingZipCode($arrPeriodicInfo['shipping_zipcode'])
                        ->setShippingAddr01($arrPeriodicInfo['shipping_addr01'])
                        ->setShippingAddr02($arrPeriodicInfo['shipping_addr02'])
                        ->setShippingDeliveryName($arrPeriodicInfo['shipping_delivery_name'])
                        ->setShippingDeliveryTime($arrPeriodicInfo['shipping_delivery_time'])
                        ->setDeliveryId($arrPeriodicInfo['delivery_id'])
                        ->setTimeId($arrPeriodicInfo['time_id'])
                        ->setFeeId($arrPeriodicInfo['fee_id'])
                        ->setSubtotal($arrPeriodicInfo['subtotal'])
                        ->setDiscount($arrPeriodicInfo['discount'])
                        ->setDeliveryFeeTotal($arrPeriodicInfo['delivery_fee_total'])
                        ->setCharge($arrPeriodicInfo['charge'])
                        ->setTax($arrPeriodicInfo['tax'])
                        ->setTotal($arrPeriodicInfo['total'])
                        ->setPaymentTotal($arrPeriodicInfo['payment_total'])
                        ->setProduct($shipmentItem->getProductClass()->getProduct())
                        ->setProductId($arrPeriodicInfo['product_id'])
                        ->setProductClassId($arrPeriodicInfo['product_class_id'])
                        ->setProductName($arrPeriodicInfo['product_name'])
                        ->setProductCode($arrPeriodicInfo['product_code'])
                        ->setClassName1($arrPeriodicInfo['class_name1'])
                        ->setClassName2($arrPeriodicInfo['class_name2'])
                        ->setClassCategoryName1($arrPeriodicInfo['class_category_name1'])
                        ->setClassCategoryName2($arrPeriodicInfo['class_category_name2'])
                        ->setPrice($arrPeriodicInfo['price'])
                        ->setQuantity($arrPeriodicInfo['quantity'])
                        ->setNote($arrPeriodicInfo['note'])
                        ->setBaseOrderId($arrPeriodicInfo['base_order_id'])
                        ->setBaseShippingId($arrPeriodicInfo['base_shipping_id'])
                        ->setBaseItemId($arrPeriodicInfo['base_item_id'])
                        ->setCreateDate($now)
                        ->setUpdateDate($now)
                        ->setDelFlg($arrPeriodicInfo['del_flg']);
                    $this->app['orm.em']->getConnection()->beginTransaction();
                    $this->app['orm.em']->persist($PayDesignPeriodicInfo);
                    $this->app['orm.em']->flush();
                    $this->app['orm.em']->getConnection()->commit();
                    $objMdl->printLog("定期マスタ作成完了:注文ID[".$order_id."]/商品規格ID[".$product_class_id."]");
                }
            }
        }
    }

    /*
     * 次回注文日作成処理
     */

    public function createPeriodicNextDate( $ProductPeriodicInfo, $base_date, $base_day,$PayDesignPeriodicInfo="", $next_flg = true ) {

        // 初期化・初期設定
        $now_date = new \DateTime();//現在日時
        $periodic_next_date = $base_date->format('Y-m-d H:i:s');
        $pnd_start_date = date('Y-m-01', strtotime($periodic_next_date));

        // 初期化・初期設定(次回注文日以外)

        if($PayDesignPeriodicInfo != ""){
        $interval = $PayDesignPeriodicInfo->getPeriodicInterval();             // 購入間隔
        }else{
            $interval = $ProductPeriodicInfo->getPeriodicInterval();  
        }
        $cutoff_day = $ProductPeriodicInfo->getPeriodicCutoffDay();          // 申込締切日
        $periodic_create_day = $ProductPeriodicInfo->getPeriodicCreateDay(); // 次回注文基準日
        // 次回注文日のベースフォーマットの設定
        if( !$next_flg ) { 
            if( is_null($periodic_create_day) ) {
                $pnd_base_format = 'Y-m-'.$base_day;
            } else {
                $pnd_base_format = 'Y-m-'.$periodic_create_day;
            }
        } else {
            // 2回目以降は$base_day, $base_dayを使用する（※定期マスタに設定されている情報）
            $pnd_base_format = 'Y-m-'.$base_day;
        }

        // パラメータの組み合わせを取得
        $param = 0;
        if( !$next_flg ) {
            if( is_null($cutoff_day) ) {
                if( is_null($periodic_create_day) ) {
                    $param = 1;
                } else {
                    $param = 2;
                }
            } else {
                if( is_null($periodic_create_day) ) {
                    $param = 3;
                } else {
                    $param = 4;
                }
            }

            // 申込締切日と計算基準日（日）データの比較（フラグ設定）
            $cod_flg = false;
            $base_date_d = intval(date('d', strtotime($base_date->format('Y-m-d H:i:s'))));
            if( $base_date_d > $cutoff_day ) { $cod_flg = true; }
        } else {
            // 2回目以降の次回注文日設定は申込締切日と次回注文基準日の考慮は不要
            $param = 1;
        }

        // 組み合わせ別に処理を行う
        switch($param) {
            case 1: // 申込締切日：設定なし／次回注文基準日：設定なし
                    // 作成日時から購入間隔分の設定を行う
                $periodic_next_date = date($pnd_base_format, strtotime($pnd_start_date."+".$interval." month"));
                break;
            case 2: // 申込締切日：設定なし／次回注文基準日：設定あり
            case 3: // 申込締切日：設定あり／次回注文基準日：設定なし
                    // 分岐は残しておくが、現状仕様ではこの２つの分岐は発生しない想定
                break;
            case 4: // 申込締切日：設定あり／次回注文基準日：設定あり
                // 初期設定時は購入間隔は"1"が基準となる
                if(!$next_flg) { $interval = 1; }

                if( $cutoff_day < $periodic_create_day ) {
                    if( $cod_flg ) {
                        // 計算基準日が申込締切日を超えている場合
                        $periodic_next_date = date($pnd_base_format, strtotime($pnd_start_date."+".$interval." month"));
                    } else { 
                        // 計算基準日が申込締切日を超えていない場合（同じ場合も）
                        $periodic_next_date = date($pnd_base_format, strtotime($pnd_start_date));
                    }
                } else {
                    if( $cod_flg ) {
                        // 計算基準日が申込締切日を超えている場合
                        $interval++;
                    } else { 
                        // 計算基準日が申込締切日を超えていない場合（同じ場合も）... 処理なし
                    }
                    $periodic_next_date = date($pnd_base_format, strtotime($pnd_start_date."+".$interval." month"));
                }
                break;
            default:
                break;
        }

        // 日付の妥当性のチェックと調整(日付が矛盾する場合は対象月の末日に設定)
        list($pnd_res_y, $pnd_res_m, $pnd_res_d) = explode('-', $periodic_next_date);
        if(!checkdate($pnd_res_m, $pnd_res_d, $pnd_res_y)) {
            $periodic_next_date = date('Y-m-d', strtotime('last day of '.$pnd_res_y.'-'.$pnd_res_m)); }

        // 戻り値は日付型に変換
        return new \DateTime($periodic_next_date);
    }

    /*
     * 定期マスターによる定期注文作成処理
     */
    public function makePeriodicOrder($PeriodicInfos)
    {
        $CreateResult = array();
        $ok_html = "<strong>成功</strong>"; 
        $ng_html = "<strong><span class=\"text-danger\">失敗</span></strong>"; 

        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $userSettings = $objMdl->getUserSettings();

        if($PeriodicInfos) {
            $ok_count = 0 ;
            $ng_count = 0 ;
            $objMdl->printLog("定期マスターによる定期注文作成処理開始：対象注文数(".count($PeriodicInfos).")");
            $this->app['orm.em']->getConnection()->setAutoCommit(true);
            foreach( $PeriodicInfos as $PeriodicInfo ) {
                $err_flg = false;

                // 商品情報の取得
                $Product = $PeriodicInfo->getProduct();

                // 商品規格情報の取得
                $ProductClass = $this->app['eccube.repository.product_class']->find($PeriodicInfo->getProductClassId());

                // 注文情報の初期化
                $OrderStatus =
                    $this->app['eccube.repository.order_status']->find($this->app['config']['order_processing']);
                $Order = new \Eccube\Entity\Order($OrderStatus);

                // 注文情報の設定(会員情報関連)
                $Order = $this->setCustomerInfo($Order, $PeriodicInfo);
                if( !is_null($Order) ) {
                    // お支払い情報の設定
                    $Payment = $this->app['eccube.repository.payment']->find($PeriodicInfo->getPaymentId());
                    $Order->setPayment($Payment);
                    $Order->setPaymentMethod($Payment->getMethod());
                    $Order->setCharge($Payment->getCharge());
                    // DB保存
                    $this->app['orm.em']->persist($Order);
                } else {
                    $err_flg = true;
                    $objMdl->printLog("注文情報の作成に失敗しました（定期情報ID：".$PeriodicInfo->getId()."）。");
                }

                // お届け先情報の初期化と設定
                if( !$err_flg ) {
                    $Shipping = $this->setShippingInfo($Order, $PeriodicInfo, $ProductClass); }
                if( !is_null($Shipping) ) {
                    // DB保存
                    $this->app['orm.em']->persist($Shipping);
                    $Order->addShipping($Shipping);
                } else {
                    $err_flg = true;
                    $objMdl->printLog("お届け先情報の作成に失敗しました（定期情報ID：".$PeriodicInfo->getId()."）。");
                }

                // 注文詳細の初期化と設定
                if( !$err_flg ) {
                    $OrderDetail = $this->setOrderDetailInfo($Order, $PeriodicInfo, $Product, $ProductClass); }
                if(!is_null($OrderDetail) ) {
                    // DB保存
                    $this->app['orm.em']->persist($OrderDetail);
                    $Order->addOrderDetail($OrderDetail);
                } else {
                    $err_flg = true;
                    $objMdl->printLog("注文詳細情報の作成に失敗しました（定期情報ID：".$PeriodicInfo->getId()."）。");
                }

                // 配送商品情報の作成
                if( !$err_flg ) {
                    $ShipmentItem = $this->setShipmentItemInfo($Order, $Shipping, $PeriodicInfo, $Product, $ProductClass); }
                if( !is_null($ShipmentItem) ) {
                    // DB保存
                    $this->app['orm.em']->persist($ShipmentItem);
                    $Shipping->addShipmentItem($ShipmentItem);
                } else {
                    $err_flg = true;
                    $objMdl->printLog("配送商品情報の作成に失敗しました（定期情報ID：".$PeriodicInfo->getId()."）。");
                }

                // 注文金額関連の設定
                $Order = $this->setOrderCalc($Order, $Shipping, $PeriodicInfo);

                // 全て問題なければ、DB保存を実行
                if( !$err_flg ) {
                    // DB保存実行
                    $this->app['orm.em']->flush();

                    /*********************************************************************
                     *
                     * 2017.1 EC-CUBEのバージョンアップにより、このタイミングでの作成はNGとなった。
                     *        従って、ここではコメントアウト（未使用）とする。
                     *
                    // 拡張受注情報の作成( setCreditUidCardSelectFlg は固定で"1")
                    $PaydesignOrderInfo = new \Plugin\PayDesign\Entity\PayDesignOrderInfo() ;
                    $PaydesignOrderInfo
                        ->setOrderId($Order->getId())
                        ->setSid($userSettings['prefix_word'].$Order->getId())
                        ->setOrderKind($this->app['config']['PayDesign']['const']['PDP_ORDER_KIND_NORMAL'])
                        ->setCreditUidCardSelectFlg(1)
                        ->setCheckCount(0);
                    // 登録・更新処理実行
                    $this->app['orm.em']->persist($PaydesignOrderInfo);
                    $this->app['orm.em']->flush();
                    *********************************************************************/

                    // 使用する定期用カード情報の取得
                    $PaydesignCardInfo = $this->getPeriodicCardInfo($Order);
                    if( !is_null($PaydesignCardInfo) ) {
                        $card_kind_word = "定期注文用";

                        // 現行仕様ではこの処理は不要になる（エラー解析用に一旦残す）
                        if( $PaydesignCardInfo->getCardKind() == $this->app['config']['PayDesign']['const']['PDP_CARD_KIND_NORMAL'] ) { $objMdl->printLog("定期用の決済カード情報が存在しなかったため、通常カード情報を使用します（定期情報ID：".$PeriodicInfo->getId()."）。"); $card_kind_word = "通常注文用"; }

                        // 決済処理実行(第3引数は定期購入フラグ、第4引数は確定パラメータフラグ)
                        $err_message = "";
                        $err_message = $this->execSettlement($Order, $PaydesignCardInfo, true, true);

                        if( $err_message == "" ) {
                            // 決済エラーなし
                            // 注文完了メールを送信
                            $this->app['eccube.service.mail']->sendOrderMail($Order);
                            if( isset($_SERVER['SHELL']) ) {
                                // Cron(バッチ)からの実行の場合のみメッセージスプールをフラッシュする必要がある
                                $this->app['swiftmailer.spooltransport']->getSpool()->flushQueue($this->app['swiftmailer.transport']); }
                            // 変更前の次回注文日を取得
                            $old_periodic_next_date = $PeriodicInfo->getPeriodicNextDate();
                            // 次回注文日の更新
                            $periodic_next_date = $this->updatePeriodicNextDate($PeriodicInfo->getId(), $Order);
                            // 次回注文日の変更ログを出力
                            $objMdl->printLog("次回注文日を変更しました（変更前：".$old_periodic_next_date->format('Y-m-d')."，変更後：".$periodic_next_date->format('Y-m-d')."）。");
                        } else {
                            // 決済エラーあり
                            $err_flg = true;
                            // エラーログ出力
                            $objMdl->printLog("定期注文カード決済でエラーが発生しました。（定期情報ID：".$PeriodicInfo->getId()."，注文ID：".$Order->getId()."）");
                            // 店舗側にエラーメールを送る
                            //$this->setPeriodicErrMail($Order, $PeriodicInfo, $err_message, $card_kind_word, $PaydesignOrderInfo->getSid());
                            $this->setPeriodicErrMail($Order, $PeriodicInfo, $err_message, $card_kind_word, $this->const['PDP_CARD_PREFIX_UID'].$userSettings['prefix_word'].$Order->getId());
                            if( isset($_SERVER['SHELL']) ) {
                                // Cron(バッチ)からの実行の場合のみメッセージスプールをフラッシュする必要がある
                                $this->app['swiftmailer.spooltransport']->getSpool()->flushQueue($this->app['swiftmailer.transport']); }
                            // 注文情報をキャンセル（同時に削除）
                            $this->setCancelOrder($Order);
                        }
                    } else {
                        // 決済エラーあり
                        $err_flg = true;
                        //エラーログ出力
                        $err_message = "決済用カード情報の取得に失敗しました（定期情報ID：".$PeriodicInfo->getId()."）。";
                        $objMdl->printLog($err_message);
                        // 店舗側にエラーメールを送る
                        $this->setPeriodicErrMail($Order, $PeriodicInfo, $err_message, $card_kind_word, "");
                        if( isset($_SERVER['SHELL']) ) {
                            // Cron(バッチ)からの実行の場合のみメッセージスプールをフラッシュする必要がある
                            $this->app['swiftmailer.spooltransport']->getSpool()->flushQueue($this->app['swiftmailer.transport']); }
                        // 注文情報をキャンセル（同時に削除）
                        $this->setCancelOrder($Order);
                    }
                }

                // 作成結果を保存
                if( !$err_flg ) {
                    $ok_count++;
                    $CreateResult[$PeriodicInfo->getId()] = $ok_html;
                } else { 
                    $ng_count++;
                    $CreateResult[$PeriodicInfo->getId()] = $ng_html;
                }
            }
            $this->app['orm.em']->getConnection()->setAutoCommit(false);
            $objMdl->printLog("定期マスターによる定期注文作成処理終了");
            $objMdl->printLog("作成件数：".$ok_count);
            $objMdl->printLog("失敗件数：".$ng_count);
        }

        return $CreateResult;
    }

    // 定期注文用会員情報の設定
    private function setCustomerInfo($BaseOrder, $PeriodicInfo)
    {
        // 初期設定
        $Order = $BaseOrder;

        // 会員情報の設定
        $Order->setCustomer($PeriodicInfo->getCustomer());
        $Order->setPref($PeriodicInfo->getCustomer()->getPref());
        $Order->setSex($PeriodicInfo->getCustomer()->getSex());
        $Order->setJob($PeriodicInfo->getCustomer()->getJob());
        if( $PeriodicInfo->getDeviceTypeId() ) {
            $Order->setDeviceType($this->app['eccube.repository.master.device_type']->find($PeriodicInfo->getDeviceTypeId())); }
        $Order->setMessage($PeriodicInfo->getMessage());
        $Order->setName01($PeriodicInfo->getCustomer()->getName01());
        $Order->setName02($PeriodicInfo->getCustomer()->getName02());
        $Order->setKana01($PeriodicInfo->getCustomer()->getKana01());
        $Order->setKana02($PeriodicInfo->getCustomer()->getKana02());
        $Order->setCompanyName($PeriodicInfo->getCustomer()->getCompanyName());
        $Order->setEmail($PeriodicInfo->getCustomer()->getEmail());
        $Order->setTel01($PeriodicInfo->getCustomer()->getTel01());
        $Order->setTel02($PeriodicInfo->getCustomer()->getTel02());
        $Order->setTel03($PeriodicInfo->getCustomer()->getTel03());
        $Order->setFax01($PeriodicInfo->getCustomer()->getFax01());
        $Order->setFax02($PeriodicInfo->getCustomer()->getFax02());
        $Order->setFax03($PeriodicInfo->getCustomer()->getFax03());
        $Order->setZip01($PeriodicInfo->getCustomer()->getZip01());
        $Order->setZip02($PeriodicInfo->getCustomer()->getZip02());
        $Order->setZipcode($PeriodicInfo->getCustomer()->getZipcode());
        $Order->setAddr01($PeriodicInfo->getCustomer()->getAddr01());
        $Order->setAddr02($PeriodicInfo->getCustomer()->getAddr02());
        $Order->setBirth($PeriodicInfo->getCustomer()->getBirth());

        return $Order;
    }

    // 定期注文用金額計算処理
    private function setOrderCalc($BaseOrder, $Shipping, $PeriodicInfo)
    {
        // 初期設定
        $Order = $BaseOrder;
        $BaseInfo = $this->app['eccube.repository.base_info']->get();

        // 配送料合計金額
        $Order->setDeliveryFeeTotal($Shipping->getShippingDeliveryFee());

        // 小計
        $subTotal = $Order->calculateSubTotal();
        $Order->setSubTotal($subTotal);

        // 配送料無料条件(合計金額)
        $deliveryFreeAmount = NULL;
        $deliveryFreeAmount = $BaseInfo->getDeliveryFreeAmount();
        if (!is_null($deliveryFreeAmount)) {
            // 合計金額が設定金額以上であれば送料無料
            if ($Order->getSubTotal() >= $deliveryFreeAmount) {
                $Order->setDeliveryFeeTotal(0);
                // お届け先情報の配送料も0にセット
                $shippings = $Order->getShippings();
                foreach ($shippings as $Shipping) {
                    $Shipping->setShippingDeliveryFee(0);
                }
            }
        }

        // 配送料無料条件(合計数量)
        $deliveryFreeQuantity = NULL;
        if (!is_null($deliveryFreeQuantity)) {
            // 合計数量が設定数量以上であれば送料無料
            if ($Order->calculateTotalQuantity() >= $deliveryFreeQuantity) {
                $Order->setDeliveryFeeTotal(0);
                // お届け先情報の配送料も0にセット
                $shippings = $Order->getShippings();
                foreach ($shippings as $Shipping) {
                    $Shipping->setShippingDeliveryFee(0);
                }
            }
        }

        // 消費税のみの小計
        $tax = $Order->calculateTotalTax();
        $Order->setTax($tax);

        // 合計
        $total = $subTotal + $Order->getCharge() + $Order->getDeliveryFeeTotal();
        $Order->setTotal($total);

        // お支払い合計
        $Order->setPaymentTotal($total);

        return $Order;
    }

    // 定期注文用商品情報（OrderDetail）の初期化と設定
    private function setOrderDetailInfo($Order, $PeriodicInfo, $Product, $ProductClass)
    {
        // 初期設定
        $OrderDetail = new \Eccube\Entity\OrderDetail();

        // 税率の取得
        $TaxRule = $this->app['eccube.repository.tax_rule']->getByRule($Product, $ProductClass);

        // 注文詳細（商品）情報の設定
        $OrderDetail
            ->setOrder($Order)
            ->setProduct($Product)
            ->setProductClass($ProductClass)
            ->setProductName($PeriodicInfo->getProductName())
            ->setProductCode($PeriodicInfo->getProductCode())
            ->setPrice($PeriodicInfo->getPrice())
            ->setQuantity($PeriodicInfo->getQuantity())
            ->setTaxRule($TaxRule->getCalcRule()->getId())
            ->setTaxRate($TaxRule->getTaxRate());

        $ClassCategory1 = $ProductClass->getClassCategory1();
        if (!is_null($ClassCategory1)) {
            $OrderDetail->setClasscategoryName1($ClassCategory1->getName());
            $OrderDetail->setClassName1($ClassCategory1->getClassName()->getName());
        }

        $ClassCategory2 = $ProductClass->getClassCategory2();
        if (!is_null($ClassCategory2)) {
            $OrderDetail->setClasscategoryName2($ClassCategory2->getName());
            $OrderDetail->setClassName2($ClassCategory2->getClassName()->getName());
        }

        return $OrderDetail;
    }

    // 定期注文用お届け先情報の初期化と設定
    private function setShippingInfo($Order, $PeriodicInfo, $ProductClass)
    {
        // 初期設定
        $Shipping = new \Eccube\Entity\Shipping();

        // お届け先情報の設定
        $ShippingPref = $this->app['eccube.repository.master.pref']->find($PeriodicInfo->getShippingPref());
        $Shipping
            ->setOrder($Order)
            ->setName01($PeriodicInfo->getShippingName01())
            ->setName02($PeriodicInfo->getShippingName02())
            ->setKana01($PeriodicInfo->getShippingKana01())
            ->setKana02($PeriodicInfo->getShippingKana02())
            ->setCompanyName($PeriodicInfo->getShippingCompanyName())
            ->setTel01($PeriodicInfo->getShippingTel01())
            ->setTel02($PeriodicInfo->getShippingTel02())
            ->setTel03($PeriodicInfo->getShippingTel03())
            ->setFax01($PeriodicInfo->getShippingFax01())
            ->setFax02($PeriodicInfo->getShippingFax02())
            ->setFax03($PeriodicInfo->getShippingFax03())
            ->setZip01($PeriodicInfo->getShippingZip01())
            ->setZip02($PeriodicInfo->getShippingZip02())
            ->setZipCode($PeriodicInfo->getShippingZip01() . $PeriodicInfo->getShippingZip01())
            ->setPref($ShippingPref)
            ->setAddr01($PeriodicInfo->getShippingAddr01())
            ->setAddr02($PeriodicInfo->getShippingAddr02())
            ->setDelFlg(0);

        // 配送料金関係
        $Delivery = $this->app['eccube.repository.delivery']->find($PeriodicInfo->getDeliveryId());
        $deliveryFee = $this->app['eccube.repository.delivery_fee']->findOneBy(array('Delivery' => $Delivery, 'Pref' => $ShippingPref));
        $Shipping->setDelivery($Delivery);
        $Shipping->setDeliveryFee($deliveryFee);

        // 商品ごとの配送料合計
        $productDeliveryFeeTotal = $ProductClass->getDeliveryFee() * $PeriodicInfo->getQuantity();
        $Shipping->setShippingDeliveryFee($deliveryFee->getFee() + $productDeliveryFeeTotal);
        $Shipping->setShippingDeliveryName($Delivery->getName());

        return $Shipping;
    }

    // 定期注文用配送商品情報の初期化と設定
    private function setShipmentItemInfo($Order, $Shipping, $PeriodicInfo, $Product, $ProductClass)
    {
        // 初期設定
        $ShipmentItem = new \Eccube\Entity\ShipmentItem();

        // 配送商品情報の設定
        $ShipmentItem
            ->setShipping($Shipping)
            ->setOrder($Order)
            ->setProductClass($ProductClass)
            ->setProduct($Product)
            ->setProductName($Product->getName())
            ->setProductCode($ProductClass->getCode())
            ->setPrice($PeriodicInfo->getPrice())
            ->setQuantity($PeriodicInfo->getQuantity());

        $ClassCategory1 = $ProductClass->getClassCategory1();
        if (!is_null($ClassCategory1)) {
            $ShipmentItem->setClasscategoryName1($ClassCategory1->getName());
            $ShipmentItem->setClassName1($ClassCategory1->getClassName()->getName());
        }

        $ClassCategory2 = $ProductClass->getClassCategory2();
        if (!is_null($ClassCategory2)) {
            $ShipmentItem->setClasscategoryName2($ClassCategory2->getName());
            $ShipmentItem->setClassName2($ClassCategory2->getClassName()->getName());
        }

        return $ShipmentItem;
    }

    // 定期注文作成時に使用する定期用カード情報の取得
    private function getPeriodicCardInfo($Order)
    {
        $PaydesignCardInfo = NULL;

        // 定期用のカード情報の取得
        $PaydesignCardInfo =
            $this->app['eccube.plugin.paydesign.repository.paydesign_card_info']->findOneBy(
                array( 'customer_id' => $Order->getCustomer()->getId(),
                       'card_kind'   => $this->app['config']['PayDesign']['const']['PDP_CARD_KIND_PERIODIC'],
            ));

        /*
        // 存在しなければ、通常カード情報の取得を試みる => 存在しない場合はエラーにする
        if( is_null($PaydesignCardInfo) ) {
            $PaydesignCardInfo =
                $this->app['eccube.plugin.paydesign.repository.paydesign_card_info']->findOneBy(
                    array( 'customer_id' => $Order->getCustomer()->getId(),
                           'card_kind'   => $this->app['config']['PayDesign']['const']['PDP_CARD_KIND_NORMAL'],
                ));
        }
        */

        return  $PaydesignCardInfo;
    }

    // （定期注文作成成功時）次回注文日更新処理
    private function updatePeriodicNextDate($periodic_id, $Order)
    {
        $PayDesignPeriodicInfo = NULL;
        $periodic_next_date = NULL;

        $objMdl =& PluginUtil::getInstance($this->app);

        // 対象の定期情報を取得する
        $PayDesignPeriodicInfo = $this->app['eccube.plugin.paydesign.repository.paydesign_periodic_info']->find($periodic_id);

        if( !is_null($PayDesignPeriodicInfo) ) {

            // 対象定期情報に紐づいた定期商品の情報の取得する
            $ProductPeriodicInfo = $this->app['eccube.plugin.paydesign.repository.paydesign_product_periodic_info']->find($PayDesignPeriodicInfo->getProductClassId());

            if( !is_null($ProductPeriodicInfo) ) {

                /*
                 * 【実行日のタイミング・考え方について】
                 * 基準を次回注文日に統一する。
                 * この変更により未来の日付が対象でも、そのまま未来の次回注文日に更新される。
                 */
                 $periodic_next_date = $PayDesignPeriodicInfo->getPeriodicNextDate();
                 $pnd_day = $PayDesignPeriodicInfo->getPeriodicCreateDay();

                // 次回注文日を更新
                $periodic_next_date = $this->createPeriodicNextDate($ProductPeriodicInfo,
                                                                     $periodic_next_date, $pnd_day,$PayDesignPeriodicInfo);

                // 次回注文日を更新する
                $PayDesignPeriodicInfo
                    ->setPeriodicNextDate($periodic_next_date)
                    ->setUpdateDate($now);

                // 更新処理実行
                $this->app['orm.em']->persist($PayDesignPeriodicInfo);
                $this->app['orm.em']->flush();
            }
        }

        return $periodic_next_date;
    }

    // （定期注文作成失敗時）店舗側にエラーメールを送る
    private function setPeriodicErrMail($Order, $PeriodicInfo, $err_message, $card_kind, $sid = "")
    {
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();

        // メール送信用データの作成
        $BaseInfo = $this->app['eccube.repository.base_info']->get();
        $subject = '[' . $BaseInfo->getShopName() . '] ' . $this->app['config']['PayDesign']['const']['PDP_PERIODIC_CREATE_ERR_SUBJECT'];
        $to_mailaddr = $BaseInfo->getEmail01();
        $formData = array(
            'periodic_id'        => $PeriodicInfo->getId(),
            'periodic_next_date' => $PeriodicInfo->getPeriodicNextDate()->format('Y-m-d H:i:s'),
            'shop_name'          => $BaseInfo->getShopName(),
            'customer_id'        => $Order->getCustomer()->getId(),
            'name01'             => $Order->getCustomer()->getName01(),
            'name02'             => $Order->getCustomer()->getName02(),
            'kana01'             => $Order->getCustomer()->getKana01(),
            'kana02'             => $Order->getCustomer()->getKana02(),
            'email'              => $Order->getCustomer()->getEmail(),
            'card_kind'          => $card_kind,
            'order_id'           => $Order->getId(),
            'order_sid'          => $sid,
            'err_message'        => $err_message,
        );
        $body = $this->app->renderView("PayDesign/View/mail_template/periodic_create_err.twig", array(
            'data' => $formData,
        ));

        // メール送信(第4引数のfalse は Bcc を送信しない。このメールはToで店舗におくるだけ)
        $objMdl->sendMail( $subject, $to_mailaddr, $body, false );

        return;
    }

    // （定期注文作成失敗時）注文情報キャンセル／削除処理
    private function setCancelOrder($Order)
    {
        $objMdl =& PluginUtil::getInstance($this->app);
        $objMdl->install();
        $objMdl->printLog("（定期注文作成失敗時）注文情報キャンセル／削除処理 開始");
        $objMdl->printLog("対象注文ID：".$Order->getId());

        $note = "";
        $note = "定期注文作成処理失敗によりキャンセル／削除処理されました";

        // setDelFlg(1) は対象注文の論理削除をしている
        $now = new \DateTime();
        $OrderCancelStatus = $this->app['eccube.repository.order_status']->find($this->app['config']['order_cancel']);
        $TargetOrder = $Order;
        $TargetOrder
            ->setOrderStatus($OrderCancelStatus)
            ->setNote($note)
            ->setDelFlg(1)
            ->setUpdateDate($now);

        // 更新処理実行
        $this->app['orm.em']->persist($TargetOrder);
        $this->app['orm.em']->flush();

        $objMdl->printLog("（定期注文作成失敗時）注文情報キャンセル／削除処理 終了");
    }

    /*
     * 引数の会員に紐づいた全ての定期マスターの削除処理
     * マイページの退会処理と会員管理画面の削除処理で使用
     */
    public function deletePeriodicInfoAll($customer_id)
    {
        if($customer_id)
        {
            // 引数の会員に紐づいた全ての定期マスターを取得
            $PayDesignPeriodicInfos = $this->app['eccube.plugin.paydesign.repository.paydesign_periodic_info']->findBy(array('customer_id'=>$customer_id));
            if($PayDesignPeriodicInfos)
            {
                // 取得した定期マスターの定期ステータス（退会）、削除フラグをセットする
                foreach( $PayDesignPeriodicInfos as $PayDesignPeriodicInfo ) {
                    $now = new \DateTime();
                    $PayDesignPeriodicInfo
                        ->setPeriodicStatus(4)
                        ->setUpdateDate($now)
                        ->setDelFlg(1);
                    $this->app['orm.em']->persist($PayDesignPeriodicInfo);
                }
                $this->app['orm.em']->flush();
            }
        }
    }
}
